///////////////////////////////////////////////////////////////////////////////
// Borderline class
//
// Authors: R. V. de Almeira Jr and H. J. Bortolossi.
//
// Created: 06/12/2005 -- Initial design and implementation.
//
// Last update: 23/12/2005
///////////////////////////////////////////////////////////////////////////////
package earth;

///////////////////////////////////////////////////////////////////////////////
// Native JAVA imports
///////////////////////////////////////////////////////////////////////////////
import java.awt.Color;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.lang.ClassLoader;

///////////////////////////////////////////////////////////////////////////////
// JavaView imports
///////////////////////////////////////////////////////////////////////////////
import jv.geom.PgPolygonSet;

public class Borderline extends PgPolygonSet
{
   protected String basePath;
   protected String structureName;
   final int DIMENSION = 3;

   // Constructor
   public Borderline(String name, String path)
   {
      super();
      this.setDimOfVertices(DIMENSION);
      this.structureName = name;
      this.basePath = path;

      this.loadBorderLine(path);
   }; // End of Borderline()

   // Reads borderline data from file
   protected void loadBorderLine(String path)
   {
      BufferedReader f;
      BorderlineFileParser fp;
      try
      {
         f = new BufferedReader(new InputStreamReader(getClass().getResourceAsStream(path)));
         fp = new BorderlineFileParser(f, this);
      }
      catch (Exception e)
      {
         System.out.println("Error: unable to read " + path + " file ...");
      };
   }; // End of loadBorderLine()

   public double getThickness()
   {
      return this.getGlobalPolygonSize();
   };

   public void setThickness(double thickness)
   {
      this.setGlobalPolygonSize(thickness);
      this.update(this);
   };

   public Color getColor()
   {
     return this.getGlobalPolygonColor();
   };

   public void setColor(Color c)
   {
      this.setGlobalPolygonColor(c);
      this.update(this);
   };

   public boolean update(Object o)
   {
      return(super.update(o));
   };

   public String toString()
   {
      return(structureName);
   };
}; // End of Borderline


