///////////////////////////////////////////////////////////////////////////////
// Earth Applet class
//
// Authors: R. V. de Almeira Jr, J. Hurrelmann, K. Polthier and
//          H. J. Bortolossi.
//
// Created: 06/12/2005 -- Initial design and implementation.
// Updated: 10/05/2006 -- Earth's texture file added.
//
// Last update: 10/05/2006
///////////////////////////////////////////////////////////////////////////////
package earth;

///////////////////////////////////////////////////////////////////////////////
// Native JAVA imports
///////////////////////////////////////////////////////////////////////////////
import java.applet.Applet;
import java.applet.AppletContext;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Rectangle;

///////////////////////////////////////////////////////////////////////////////
// JavaView imports
///////////////////////////////////////////////////////////////////////////////
import jv.object.*;
import jv.viewer.PvViewer;
import jv.project.PvDisplayIf;

///////////////////////////////////////////////////////////////////////////////
// Earth applet
///////////////////////////////////////////////////////////////////////////////
public class PaEarth extends Applet
{
   public Frame m_frame = null; // Frame if run standalone, null if run  as
                                // applet.


   protected PvViewer m_viewer; // 3D-viewer window for graphics output and
                                // which is embedded into the applet.


   ////////////////////////////////////////////////////////////////////////////
   // Applet parameters: {"Name", "Typ", "Default value", "Description"}
   ////////////////////////////////////////////////////////////////////////////
   protected String [][] m_parm =
   {
      {"Console", "String", "Hide", "Show/Hide VGP-console for debugging"},
      {"Control", "String", "Hide", "Show/Hide control panel"},
      {"Frame", "String", "Hide", "Show/Hide frame around applet"},
      {"Panel",   "String", "Project",
       "Name of initial panel if control panel is showing"}
   };

   ////////////////////////////////////////////////////////////////////////////
   // Interface used by design tools to show properties of applet. This method
   // returns a list of string arrays, each of  length  4  rather  than  3  as
   // suggest by Java. The additional string at third  position  contains  the
   // value of the parameter.
   ////////////////////////////////////////////////////////////////////////////
   public String[][] getParameterInfo()
   {
      return m_parm;
   }; // End of getParameterInfo()

   ////////////////////////////////////////////////////////////////////////////
   // Applet's interface to inform about author, version, and copyright
   ////////////////////////////////////////////////////////////////////////////
   public String getAppletInfo()
   {
      return("Authors: " + "R. V. de Almeira Jr e H. J. Bortolossi" + "\n" +
             "Version: " + "1.00 (12/23/2005)" + "\n");
   }; // End of getAppletInfo()

   ////////////////////////////////////////////////////////////////////////////
   // Configure and initialize the viewer,  load  system  and  user  projects.
   // One of the user projects must be selected here.
   ////////////////////////////////////////////////////////////////////////////
   public void init()
   {
      // Create viewer for viewing 3d geometries
      m_viewer = new PvViewer(this, m_frame);

      // Get applet parameters from the source html file
      boolean viewGlobe;
      if (m_viewer.getParameter("viewGlobe").equals("true"))
      {
         viewGlobe = true;
      }
      else
      {
         viewGlobe = false;
      };

      boolean viewEarthSurface;
      if (m_viewer.getParameter("viewEarthSurface").equals("true"))
      {
         viewEarthSurface = true;
      }
      else
      {
         viewEarthSurface = false;
      };

      boolean viewCoastline;
      if (m_viewer.getParameter("viewCoastline").equals("true"))
      {
         viewCoastline = true;
      }
      else
      {
         viewCoastline = false;
      };

      boolean viewAmericaBnd;
      if (m_viewer.getParameter("viewAmericaBnd").equals("true"))
      {
         viewAmericaBnd = true;
      }
      else
      {
         viewAmericaBnd = false;
      };

      boolean viewAfricaBnd;
      if (m_viewer.getParameter("viewAfricaBnd").equals("true"))
      {
         viewAfricaBnd = true;
      }
      else
      {
         viewAfricaBnd = false;
      };

      boolean viewAsiaBnd;
      if (m_viewer.getParameter("viewAsiaBnd").equals("true"))
      {
         viewAsiaBnd = true;
      }
      else
      {
         viewAsiaBnd = false;
      };

      boolean viewEuropeBnd;
      if (m_viewer.getParameter("viewEuropeBnd").equals("true"))
      {
         viewEuropeBnd = true;
      }
      else
      {
         viewEuropeBnd = false;
      };

      boolean viewOceaniaBnd;
      if (m_viewer.getParameter("viewOceaniaBnd").equals("true"))
      {
         viewOceaniaBnd = true;
      }
      else
      {
         viewOceaniaBnd = false;
      };

      boolean viewGreenwich;
      if (m_viewer.getParameter("viewGreenwich").equals("true"))
      {
         viewGreenwich = true;
      }
      else
      {
         viewGreenwich = false;
      };

      boolean viewEquator;
      if (m_viewer.getParameter("viewEquator").equals("true"))
      {
         viewEquator = true;
      }
      else
      {
         viewEquator = false;
      };

      boolean viewParalellAngle;
      if (m_viewer.getParameter("viewParalellAngle").equals("true"))
      {
         viewParalellAngle = true;
      }
      else
      {
         viewParalellAngle = false;
      };

      boolean viewMeridianAngle;
      if (m_viewer.getParameter("viewMeridianAngle").equals("true"))
      {
         viewMeridianAngle = true;
      }
      else
      {
         viewMeridianAngle = false;
      };

      boolean viewCancer;
      if (m_viewer.getParameter("viewCancer").equals("true"))
      {
         viewCancer = true;
      }
      else
      {
        viewCancer = false;
      };

      boolean viewCapricorn;
      if (m_viewer.getParameter("viewCapricorn").equals("true"))
      {
         viewCapricorn = true;
      }
      else
      {
         viewCapricorn = false;
      };

      boolean viewArctic;
      if (m_viewer.getParameter("viewArctic").equals("true"))
      {
         viewArctic = true;
      }
      else
      {
         viewArctic = false;
      };

      boolean viewAntarctic;
      if (m_viewer.getParameter("viewAntarctic").equals("true"))
      {
         viewAntarctic = true;
      }
      else
      {
         viewAntarctic = false;
      };

      double latIni =
                 Double.valueOf(m_viewer.getParameter("latIni")).doubleValue();
      double lonIni =
                 Double.valueOf(m_viewer.getParameter("lonIni")).doubleValue();

      double globeTransparency =
      Double.valueOf(m_viewer.getParameter("globeTransparency")).doubleValue();

      int tabActive =
                Integer.valueOf(m_viewer.getParameter("tabActive")).intValue();

      int idiom =
                Integer.valueOf(m_viewer.getParameter("idiom")).intValue();

      String initialLocation = m_viewer.getParameter("initialLocation");

      // Create and load a project
      PjEarth m_p = new PjEarth(this.getAppletContext(),
//                              this.getCodeBase().toString(),
                                this.getCodeBase().toString() + "earth/",
                                viewGlobe,
                                viewEarthSurface,
                                viewCoastline,
                                viewAmericaBnd,
                                viewAfricaBnd,
                                viewAsiaBnd,
                                viewEuropeBnd,
                                viewOceaniaBnd,
                                viewGreenwich,
                                viewEquator,
                                viewParalellAngle,
                                viewMeridianAngle,
                                viewCancer,
                                viewCapricorn,
                                viewArctic,
                                viewAntarctic,
                                latIni,
                                lonIni,
                                globeTransparency,
                                tabActive,
				idiom,
				initialLocation);
      m_viewer.addProject(m_p);
      m_viewer.selectProject(m_p);

      // Get 3d display from viewer and add it to applet
      setLayout(new BorderLayout());
      PvDisplayIf disp = m_viewer.getDisplay();

      // DBackground color
      disp.setBackgroundColor(Color.BLACK);

      // Scale camera such that current geometries fit into display
      disp.fit();

      // Get 3d display from viewer and add it to applet
      setLayout(new BorderLayout());
      add((Component)disp, BorderLayout.CENTER);
      add(m_viewer.getPanel(PsViewerIf.PROJECT), BorderLayout.EAST);
      validate();

      // Set default viewer panel
      m_viewer.showPanel(PsViewerIf.MATERIAL);
   }; // End of init()

   ////////////////////////////////////////////////////////////////////////////
   // Standalone application support. The main() method acts as the applet's
   // entry point when it is run as a standalone application. It is  ignored
   // if the applet is run from within an HTML page.
   ////////////////////////////////////////////////////////////////////////////
   public static void main(String args[])
   {
      PaEarth va = new PaEarth();
      // Create toplevel window of application containing the applet
      Frame frame = new jv.object.PsMainFrame(va, args);
      frame.pack();
      va.m_frame = frame;
      va.init();
      va.start();
      frame.setBounds(new Rectangle(220, 5, 800, 550));
      frame.setVisible(true);
   }; // End of main()

   ////////////////////////////////////////////////////////////////////////////
   // Print info while initializing applet and viewer
   ////////////////////////////////////////////////////////////////////////////
   public void paint(Graphics g)
   {
      g.setColor(Color.blue);
      g.drawString("PaEarth Applet", 20, 40);
      g.drawString("Loading Projects ...", 20, 60);
   }; // End of paint()

   ////////////////////////////////////////////////////////////////////////////
   // Does clean-up when applet is destroyed by the browser. Here we just
   // close and dispose all our control windows.
   ////////////////////////////////////////////////////////////////////////////
   public void destroy()
   {
      m_viewer.destroy();
   }; // End of destroy()

   ////////////////////////////////////////////////////////////////////////////
   // Start viewer, e.g. start animation if requested
   ////////////////////////////////////////////////////////////////////////////
   public void start()
   {
      m_viewer.start();
   }; // End of start()

   ////////////////////////////////////////////////////////////////////////////
   // Stop viewer, e.g. stop animation if requested
   ////////////////////////////////////////////////////////////////////////////
   public void stop()
   {
      m_viewer.stop();
   }; // End of stop()
}; // End of PaEarth class


