///////////////////////////////////////////////////////////////////////////////
// Earth Applet class
//
// Authors: R. V. de Almeira Jr, J. Hurrelmann, K. Polthier and
//          H. J. Bortolossi.
//
// Created: 06/12/2005 -- Initial design and implementation.
// Updated: 10/05/2006 -- Earth's texture file added.
//
// Last update: 10/05/2006
///////////////////////////////////////////////////////////////////////////////
package earth;

///////////////////////////////////////////////////////////////////////////////
// Native JAVA imports
///////////////////////////////////////////////////////////////////////////////
import java.applet.AppletContext;
import java.awt.Color;
import java.awt.Image;
import java.awt.Label;
import java.net.URL;

///////////////////////////////////////////////////////////////////////////////
// JavaView imports
///////////////////////////////////////////////////////////////////////////////
import jv.function.PuFunction;
import jv.geom.PgPolygon;
import jv.geom.PgElementSet;
import jv.geom.PgTexture;
import jv.objectGui.PsImage;
import jv.project.PgGeometryIf;
import jv.project.PgJvxSrc;
import jv.project.PjProject;
import jv.project.PvCameraIf;
import jv.project.PvDisplayIf;
import jv.project.PvPickListenerIf;
import jv.project.PvPickEvent;
import jvx.rsrc.JarResources;
import jvx.surface.PgDomainDescr;
import jvx.surface.PgParmSurface;
import jv.vecmath.PdVector;


///////////////////////////////////////////////////////////////////////////////
// Earth applet
///////////////////////////////////////////////////////////////////////////////
public class PjEarth extends PjProject implements PvPickListenerIf
{
   int language = 0; // Use 0 to portuguese and 1 to english

   protected AppletContext appletContext;

   protected String codebase;

   protected double globe_radius = 1.0;
   protected Borderline coastline;
   protected Borderline america_bnd;
   protected Borderline asia_bnd;
   protected Borderline europe_bnd;
   protected Borderline africa_bnd;
   protected Borderline oceania_bnd;
   protected PgParmSurface globe;
   protected PgPolygon equator;
   protected PgPolygon cancer;
   protected PgPolygon capricorn;
   protected PgPolygon arctic_circle;
   protected PgPolygon antarctic_circle;
   protected PgPolygon greenwich;
   protected PgPolygon poles;

   protected PgPolygon par_by_point;      // Parallel through picked point
   protected PgPolygon mer_by_point;      // Meridian through picked point
   protected PgPolygon picked_point;      // Picked point

   protected Label lat_l;
   protected Label lon_l;

   protected PgParmSurface par_angle_sur; // Parallel angle surface
   protected PuFunction par_angle_fun;
   protected PgDomainDescr par_angle_dom;
   protected PgParmSurface mer_angle_sur; // Meridian angle surface
   protected PuFunction mer_angle_fun;
   protected PgDomainDescr mer_angle_dom;
   protected PgPolygon ray1;              // Helps visualization of parallel
   protected PgPolygon ray2;              // and meridian angles.

   protected int globe_lat_n_of_p = 29;
   protected int globe_lon_n_of_p = 25;
   protected int parallel_n_of_p = 50;
   protected int meridian_n_of_p = 50;
   protected int parallel_angle_n_of_p = 30;
   protected int meridian_angle_n_of_p = 30;

   String[] N_trans = {"N", "N"};
   String[] S_trans = {"S", "S"};
   String[] W_trans = {"O", "W"};
   String[] E_trans = {"L", "E"};
   String[] C_trans = {"C", "C"};

   boolean viewGlobe;
   boolean viewAmericaBnd;
   boolean viewAfricaBnd;
   boolean viewAsiaBnd;
   boolean viewEuropeBnd;
   boolean viewOceaniaBnd;
   boolean viewGreenwich;
   boolean viewEquator;
   boolean viewMeridianAngle;
   boolean viewCancer;
   boolean viewCapricorn;
   boolean viewParalellAngle;
   boolean viewArctic;
   boolean viewAntarctic;
   boolean viewCoastline;
   boolean viewEarthSurface;

   double latIni;
   double lonIni;

   double globeTransparency;

   int tabActive;

   String initialLocation;

   protected int m_numULines = 30;
   protected int m_numVLines = 30;
   protected PgElementSet earth_sur;
   protected String earth_tex = "/earth/flags/surface.jpg";
   protected double defaultTrans = 0.0;

   protected PgPolygon city1_seg;
   protected PgPolygon city2_seg;
   protected PgPolygon geodesic;
   protected int geodesic_n_of_p = 50;
   protected boolean show_geodesic = false;
   protected double earth_radius =  6378; // In kilometers
   protected Label distLabel;
   protected String ndef_trans;


   //////////////////////////////////////////////////////////////////////////
   // Default constructor
   //////////////////////////////////////////////////////////////////////////
   public PjEarth(AppletContext input_appletContext,
                  String input_codebase,
                  boolean input_viewGlobe,
                  boolean input_viewEarthSurface,
                  boolean input_viewCoastline,
                  boolean input_viewAmericaBnd,
                  boolean input_viewAfricaBnd,
                  boolean input_viewAsiaBnd,
                  boolean input_viewEuropeBnd,
                  boolean input_viewOceaniaBnd,
                  boolean input_viewGreenwich,
                  boolean input_viewEquator,
                  boolean input_viewParalellAngle,
                  boolean input_viewMeridianAngle,
                  boolean input_viewCancer,
                  boolean input_viewCapricorn,
                  boolean input_viewArctic,
                  boolean input_viewAntarctic,
                  double input_latIni,
                  double input_lonIni,
                  double input_globeTransparency,
                  int input_tabActive,
		  int input_language,
		  String input_initialLocation)
   {
      // Calls parent
      super("Earth Applet");

      // Parses input paramemeters
      appletContext = input_appletContext;
      codebase = input_codebase;
      viewGlobe = input_viewGlobe;
      viewEarthSurface = input_viewEarthSurface;
      viewCoastline = input_viewCoastline;
      viewAmericaBnd = input_viewAmericaBnd;
      viewAfricaBnd = input_viewAfricaBnd;
      viewAsiaBnd = input_viewAsiaBnd;
      viewEuropeBnd = input_viewEuropeBnd;
      viewOceaniaBnd = input_viewOceaniaBnd;
      viewGreenwich = input_viewGreenwich;
      viewEquator = input_viewEquator;
      viewParalellAngle = input_viewParalellAngle;
      viewMeridianAngle = input_viewMeridianAngle;
      viewCancer = input_viewCancer;
      viewCapricorn = input_viewCapricorn;
      viewArctic = input_viewArctic;
      viewAntarctic = input_viewAntarctic;

      latIni = input_latIni;
      lonIni = input_lonIni;

      globeTransparency = input_globeTransparency;

      tabActive = input_tabActive;

      language = input_language;

      initialLocation = input_initialLocation;

      if (getClass() == PjEarth.class)
      {
         init();
      };
   }; // End of PjEarth()

   //////////////////////////////////////////////////////////////////////////
   // Initialize variables
   //////////////////////////////////////////////////////////////////////////
   public void init()
   {
      // Creates the Earth's Globe
      globe = computeSphere(globe_radius, Color.BLUE);
      globe.setTransparency(globeTransparency);
      globe.showTransparency(true);

      // Creates parallels
      equator = computeParallel(0.0, globe_radius, Color.GREEN);
      cancer = computeParallel(23.5, globe_radius, Color.GREEN);
      capricorn = computeParallel(-23.5, globe_radius, Color.GREEN);
      arctic_circle = computeParallel(66.3, globe_radius, Color.GREEN);
      antarctic_circle = computeParallel(-66.3, globe_radius, Color.GREEN);

      // Creates meridians
      greenwich = computeMeridian(0.0, globe_radius, Color.RED);

      // Creates poles
      poles = computePoles(globe_radius, Color.WHITE);

      // Creates the picked point
      picked_point = new PgPolygon(3);
      picked_point.setName("Picked Point");
      picked_point.setNumVertices(2);
      picked_point.showVertices(true);
      picked_point.setVertexColor(1, Color.WHITE);
      picked_point.showVertexColors(true);
      picked_point.setGlobalEdgeColor(Color.WHITE);
      picked_point.setVertex(0, 0.0, 0.0, 0.0);

      double lat_in_rad = latIni*Math.PI/180.0;
      double lon_in_rad = lonIni*Math.PI/180.0;
      double x = globe_radius*Math.cos(lat_in_rad)*Math.cos(lon_in_rad);
      double y = globe_radius*Math.cos(lat_in_rad)*Math.sin(lon_in_rad);
      double z = globe_radius*Math.sin(lat_in_rad);
      picked_point.setVertex(1, x, y, z);

      picked_point.getVertex(0).setName("");
      picked_point.getVertex(1).setName("");
      picked_point.setLabelColor(0, Color.WHITE);
      picked_point.setLabelColor(1, Color.WHITE);
      picked_point.setLabelSize(0, 20);
      picked_point.setLabelSize(1, 20);
      picked_point.showVertexLabels(true);
      lat_l = new Label(lat2str(latIni), Label.RIGHT);
      lon_l = new Label(lon2str(lonIni), Label.RIGHT);

      // Creates parallel passing by the picked point
      par_by_point = computeParallel(latIni, globe_radius, Color.CYAN);

      // creates meridian passing the picked point
      mer_by_point = computeMeridian(lonIni, globe_radius, Color.CYAN);

      // Creates the Earth's coastline
      coastline = new Borderline("Earth Coastline",
                                 "shorelines.txt");
      coastline.showPolygons(true); // To make the borderline visible
      coastline.setColor(Color.YELLOW);
      coastline.setThickness(1.0);

      // Creates the parallel angle
      // Setup parametrized surface functions
      double r = globe_radius;

      double cos_lon = Math.cos(lon_in_rad);
      double sin_lon = Math.sin(lon_in_rad);
      par_angle_fun = new PuFunction(2, 3);
      par_angle_fun.setName("Latitude angle");
      par_angle_fun.setExpression(0, Double.toString(r) + "*u*cos(v)*" +
                                     Double.toString(cos_lon));
      par_angle_fun.setExpression(1, Double.toString(r)  + "*u*cos(v)*" +
                                     Double.toString(sin_lon));
      par_angle_fun.setExpression(2, Double.toString(r) + "*u*sin(v)");

      // Setup parametrized surface domain discretization
      par_angle_dom = new PgDomainDescr(2);
      par_angle_dom.setName("Domain of the Parallel Angle Surface");
      if (lat_in_rad > 0.0)
      {
          par_angle_dom.setMaxSize(0, 0, globe_radius, lat_in_rad);
          par_angle_dom.setSize(0, 0, globe_radius, lat_in_rad);
      }
      else
      {
          par_angle_dom.setMaxSize(0, lat_in_rad, globe_radius, 0.0);
          par_angle_dom.setSize(0, lat_in_rad, globe_radius, 0.0);
      };
      par_angle_dom.setDiscr(2, parallel_angle_n_of_p);
      par_angle_dom.init();

      par_angle_sur = new PgParmSurface(3);
      par_angle_sur.setDomainDescr(par_angle_dom);
      par_angle_sur.setFunctionExpr(par_angle_fun);
      par_angle_sur.compute();

      par_angle_sur.setGlobalElementColor(Color.GREEN);
      par_angle_sur.showEdges(false);
      par_angle_sur.setTransparency(0.2);
      par_angle_sur.showTransparency(false);

      // Creates the meridian angle
      // Setup parametrized surface functions
      mer_angle_fun = new PuFunction(2, 3);
      mer_angle_fun.setName("Longitude Angle");
      mer_angle_fun.setExpression(0, Double.toString(r) + "*v*cos(u)");
      mer_angle_fun.setExpression(1, Double.toString(r) + "*v*sin(u)");
      mer_angle_fun.setExpression(2, Double.toString(0.0));

      // Setup parametrized surface domain discretization
      mer_angle_dom = new PgDomainDescr(2);
      mer_angle_dom.setName("Domain of the Meridian Angle");
      if (lon_in_rad > 0.0)
      {
          mer_angle_dom.setMaxSize(0, 0, lon_in_rad, globe_radius);
          mer_angle_dom.setSize(0, 0, lon_in_rad, globe_radius);
      }
      else
      {
          mer_angle_dom.setMaxSize(lon_in_rad, 0, 0, globe_radius);
          mer_angle_dom.setSize(lon_in_rad, 0, 0, globe_radius);
      };
      mer_angle_dom.setDiscr(meridian_angle_n_of_p, 2);
      mer_angle_dom.init();

      mer_angle_sur = new PgParmSurface(3);
      mer_angle_sur.setDomainDescr(mer_angle_dom);
      mer_angle_sur.setFunctionExpr(mer_angle_fun);
      mer_angle_sur.compute();

      mer_angle_sur.setGlobalElementColor(Color.RED);
      mer_angle_sur.showEdges(false);
      mer_angle_sur.setTransparency(0.2);
      mer_angle_sur.showTransparency(false);

      // Rays to help visualize parallel and meridian angles
      ray1 = new PgPolygon(3);
      ray1.setName("Ray 1");
      ray1.setNumVertices(2);
      ray1.showVertices(false);
      ray1.setVertexColor(1, Color.WHITE);
      ray1.setGlobalEdgeColor(Color.WHITE);
      ray1.setVertex(0, 0.0, 0.0, 0.0);
      ray1.setVertex(1, globe_radius, 0.0, 0.0);

      ray2 = new PgPolygon(3);
      ray2.setName("Ray 2");
      ray2.setNumVertices(2);
      ray2.showVertices(false);
      ray2.setVertexColor(1, Color.WHITE);
      ray2.setGlobalEdgeColor(Color.WHITE);
      ray2.setVertex(0, 0.0, 0.0, 0.0);
      ray2.setVertex(1, globe_radius, 0.0, 0.0);

      // Creates America's boundaries
      america_bnd = new Borderline("America Boundaries",
                                   "america.txt");
      america_bnd.showPolygons(true); // To make the borderline visible
      america_bnd.setColor(Color.LIGHT_GRAY);
      america_bnd.setThickness(1.0);

      // Creates Asia's boundaries
      asia_bnd = new Borderline("Asia Boundaries",
                                "asia.txt");
      asia_bnd.showPolygons(true); // To make the borderline visible
      asia_bnd.setColor(Color.LIGHT_GRAY);
      asia_bnd.setThickness(1.0);

      // Creates European boundaries
      europe_bnd = new Borderline("European Boundaries",
                                  "europe.txt");
      europe_bnd.showPolygons(true); // To make the borderline visible
      europe_bnd.setColor(Color.LIGHT_GRAY);
      europe_bnd.setThickness(1.0);

      // Creates African boundaries
      africa_bnd = new Borderline("African Boundaries",
                                   "africa.txt");
      africa_bnd.showPolygons(true); // To make the borderline visible
      africa_bnd.setColor(Color.LIGHT_GRAY);
      africa_bnd.setThickness(1.0);

      // Creates Australian boundaries
      oceania_bnd = new Borderline("oceania Boundaries",
                                  "oceania.txt");
      oceania_bnd.showPolygons(true); // To make the borderline visible
      oceania_bnd.setColor(Color.LIGHT_GRAY);
      oceania_bnd.setThickness(1.0);

      // Creates Earth's photo
      earth_sur = getSphere("Earth", 0.99, false);

      // Creates city1 and city2 position vectors
      city1_seg = new PgPolygon(3);
      city1_seg.setName("City 1 position");
      city1_seg.setNumVertices(2);
      city1_seg.showVertices(true);
      city1_seg.setVertexColor(1, Color.WHITE);
      city1_seg.showVertexColors(true);
      city1_seg.setGlobalEdgeColor(Color.WHITE);
      city1_seg.setVertex(0, 0.0, 0.0, 0.0);
      city1_seg.setVertex(1, globe_radius, 0.0, 0.0);
      city1_seg.setVertexColor(1, Color.BLUE);

      city1_seg.getVertex(0).setName("");
      city1_seg.getVertex(1).setName("");
      city1_seg.setLabelColor(0, Color.WHITE);
      city1_seg.setLabelColor(1, Color.WHITE);
      city1_seg.setLabelSize(0, 20);
      city1_seg.setLabelSize(1, 20);
      city1_seg.showVertexLabels(true);

      city2_seg = new PgPolygon(3);
      city2_seg.setName("City 2 position");
      city2_seg.setNumVertices(2);
      city2_seg.showVertices(true);
      city2_seg.setVertexColor(1, Color.WHITE);
      city2_seg.showVertexColors(true);
      city2_seg.setGlobalEdgeColor(Color.WHITE);
      city2_seg.setVertex(0, 0.0, 0.0, 0.0);
      city2_seg.setVertex(1, globe_radius, 0.0, 0.0);
      city2_seg.setVertexColor(1, Color.BLUE);

      city2_seg.getVertex(0).setName("");
      city2_seg.getVertex(1).setName("");
      city2_seg.setLabelColor(0, Color.WHITE);
      city2_seg.setLabelColor(1, Color.WHITE);
      city2_seg.setLabelSize(0, 20);
      city2_seg.setLabelSize(1, 20);
      city2_seg.showVertexLabels(true);

      // Creates geodesic joining the two cities over the globe
      geodesic = new PgPolygon(3);
      geodesic.setName("Geodesic");
      geodesic.setNumVertices(geodesic_n_of_p);
      geodesic.showVertices(false);
      geodesic.setGlobalEdgeColor(Color.BLUE);

      distLabel = new Label(" ", Label.CENTER);
   }; // End of init()

   //////////////////////////////////////////////////////////////////////////
   // Things to do when the class is started
   //////////////////////////////////////////////////////////////////////////
   public void start()
   {
      addGeometry(globe);
      if (viewGlobe == true)
      {
         globe.setTransparency(globeTransparency);
         globe.update(globe);
      }
      else
      {
         globe.setTransparency(1.0);
         globe.update(globe);
      };

      if (viewCoastline == true)
      {
         addGeometry(coastline);
      };

      if (viewAmericaBnd == true)
      {
         addGeometry(america_bnd);
      };

      if (viewAfricaBnd == true)
      {
         addGeometry(africa_bnd);
      };

      if (viewAsiaBnd == true)
      {
         addGeometry(asia_bnd);
      };

      if (viewEuropeBnd == true)
      {
        addGeometry(europe_bnd);
      };

      if (viewOceaniaBnd == true)
      {
        addGeometry(oceania_bnd);
      };

      if (viewGreenwich == true)
      {
         addGeometry(greenwich);
      };

      if (viewMeridianAngle == true)
      {
         addGeometry(mer_angle_sur);
         addGeometry(ray1);
      };

      if (viewCancer == true)
      {
         addGeometry(cancer);
      };

      if (viewEquator == true)
      {
         addGeometry(equator);
      };

      if (viewCapricorn == true)
      {
         addGeometry(capricorn);
      };

      if (viewParalellAngle == true)
      {
         addGeometry(par_angle_sur);
         addGeometry(ray2);
      };

      if (viewArctic == true)
      {
         addGeometry(arctic_circle);
      };

      if (viewAntarctic == true)
      {
         addGeometry(antarctic_circle);
      };

      addGeometry(poles);

      addGeometry(par_by_point);
      addGeometry(mer_by_point);
      addGeometry(picked_point);

      if (viewEarthSurface == true)
      {
         addGeometry(earth_sur);
      };

      selectGeometry(globe);

      update(this);
   }; // End of start()

   /////////////////////////////////////////////////////////////////////////
   // Update the class whenever a child has changed. Method is usually
   // invoked from the children.
   /////////////////////////////////////////////////////////////////////////
   public boolean update(Object event)
   {
      if (event == null)
      {
         return(true);
      };

      return(super.update(event));
   }; // End of update()

   //////////////////////////////////////////////////////////////////////////
   // This procedure computes the longitude and latitute (in degrees) of a
   // point taking the position p = (x, y, z) on the globe surface.
   //////////////////////////////////////////////////////////////////////////
   public PdVector pos2geo(double x, double y, double z)
   {
      double lat_in_deg;
      double lon_in_deg;

      if ((x <= 0.0) && (y == 0.0))
      {
         lon_in_deg = 180.0;
         lat_in_deg = Math.atan2(z, Math.abs(x))*180.0/Math.PI;
      }
      else
      {
         lat_in_deg = Math.atan2(z, Math.sqrt(x*x + y*y))*180.0/Math.PI;
         lon_in_deg = Math.atan2(y, x)*180/Math.PI;
      };

      return (new PdVector(lon_in_deg, lat_in_deg));
   }; // End of pos2geo()

   //////////////////////////////////////////////////////////////////////////
   // Computes a sphere with radius r centered at the origin (0.0, 0.0, 0.0)
   //////////////////////////////////////////////////////////////////////////
   protected PgParmSurface computeSphere(double r, Color c)
   {
      // Setup parametrized surface functions
      PuFunction m_function = new PuFunction(2, 3);
      m_function.setName("Globe");
      m_function.setExpression(0, Double.toString(r) + "*cos(v)*cos(u)");
      m_function.setExpression(1, Double.toString(r) + "*cos(v)*sin(u)");
      m_function.setExpression(2, Double.toString(r) + "*sin(v)");

      // Setup parametrized surface domain discretization
      PgDomainDescr m_domain = new PgDomainDescr(2);
      m_domain.setName("Globe Parametrization Domain");
      m_domain.setMaxSize(-Math.PI, -Math.PI/2, Math.PI, Math.PI/2);
      m_domain.setSize(-Math.PI, -Math.PI/2, Math.PI, Math.PI/2);
      m_domain.setDiscr(globe_lon_n_of_p, globe_lat_n_of_p);
      m_domain.init();

      // Computes parametrized surface
      PgParmSurface m_parmSurface = new PgParmSurface(3);
      m_parmSurface.setDomainDescr(m_domain);
      m_parmSurface.setFunctionExpr(m_function);
      m_parmSurface.compute();

      // Setup parametrized surface appearance
      m_parmSurface.setGlobalElementColor(c);
      m_parmSurface.showEdges(true);

      return(m_parmSurface);
   }; // End of computeSphere()

   //////////////////////////////////////////////////////////////////////////
   // This procedure computes a parallel of latitude lat between -90 and +90
   // degrees.
   //////////////////////////////////////////////////////////////////////////
   protected PgPolygon computeParallel(double lat_in_deg, double r, Color c)
   {
      int n_of_p = parallel_n_of_p;
      double lat_in_rad = lat_in_deg*Math.PI/180.0;
      PgPolygon parallel = new PgPolygon(3);
      parallel.setName("Parallel");
      parallel.setGlobalEdgeColor(c);
      parallel.setNumVertices(n_of_p + 1);
      parallel.showVertices(false);

      for (int i = 0; i <= n_of_p; i++)
      {
         double lon = -Math.PI + (1.0*i)/n_of_p*2.0*Math.PI;
         parallel.setVertex(i, r*Math.cos(lat_in_rad)*Math.cos(lon),
                               r*Math.cos(lat_in_rad)*Math.sin(lon),
                               r*Math.sin(lat_in_rad));
      };

      return(parallel);
   }; // End of computeParallel()

   //////////////////////////////////////////////////////////////////////////
   // This procedure computes a meridian of longitude lon between -180 and
   // +180 degrees.
   //////////////////////////////////////////////////////////////////////////
   protected PgPolygon computeMeridian(double lon_in_deg, double r, Color c)
   {
      int n_of_p = meridian_n_of_p;
      double lon_in_rad = lon_in_deg*Math.PI/180.0;
      PgPolygon meridian = new PgPolygon(3);
      meridian.setName("Meridian");
      meridian.setGlobalEdgeColor(c);
      meridian.setNumVertices(n_of_p + 1);
      meridian.showVertices(false);

      for (int i = 0; i <= n_of_p; i++)
      {
         double lat = -Math.PI/2.0 + (1.0*i)/n_of_p*Math.PI;
         meridian.setVertex(i, r*Math.cos(lat)*Math.cos(lon_in_rad),
                               r*Math.cos(lat)*Math.sin(lon_in_rad),
                               r*Math.sin(lat));
      };

      return (meridian);
   }; // End of computeMeridian()

   //////////////////////////////////////////////////////////////////////////
   // This procedure computes the globe's poles
   //////////////////////////////////////////////////////////////////////////
   protected PgPolygon computePoles(double r, Color c)
   {
      PgPolygon poles = new PgPolygon(3);
      poles.setName("Poles");
      poles.setNumVertices(3);
      poles.showVertices(true);
      poles.setVertexColor(0, c);
      poles.setVertexColor(2, c);
      poles.showVertexColors(true);
      poles.setVertex(0, 0.0, 0.0, +r);
      poles.setVertex(1, 0.0, 0.0, 0.0);
      poles.setVertex(2, 0.0, 0.0, -r);
      poles.getVertex(0).setName(N_trans[language]);
      poles.getVertex(1).setName(C_trans[language]);
      poles.getVertex(2).setName(S_trans[language]);
      poles.showVertexLabels(true);
      poles.setLabelColor(0, c);
      poles.setLabelColor(2, c);
      poles.setGlobalEdgeColor(Color.white);

      return (poles);
   }; // End of computePoles()

   //////////////////////////////////////////////////////////////////////////
   // Updates parallel angle (latitude) surface
   //////////////////////////////////////////////////////////////////////////
   protected void updateParallelAngle(double lat_in_rad, double lon_in_rad,
                                      double r, Color c)
   {
      par_angle_dom = new PgDomainDescr(2);
      par_angle_dom.setName("Domain of the Parallel Angle Surface");
      if (lat_in_rad > 0.0)
      {
          par_angle_dom.setMaxSize(0.0, 0.0, globe_radius, lat_in_rad);
          par_angle_dom.setSize(0.0, 0.0, globe_radius, lat_in_rad);
      }
      else
      {
          par_angle_dom.setMaxSize(0.0, lat_in_rad, globe_radius, 0.0);
          par_angle_dom.setSize(0.0, lat_in_rad, globe_radius, 0.0);
      };
      par_angle_dom.setDiscr(2, parallel_angle_n_of_p);
      par_angle_dom.init();

      double cos_lon = Math.cos(lon_in_rad);
      double sin_lon = Math.sin(lon_in_rad);
      par_angle_fun = new PuFunction(2, 3);
      par_angle_fun.setExpression(0, Double.toString(r) + "*u*cos(v)*" +
                                     Double.toString(cos_lon));
      par_angle_fun.setExpression(1, Double.toString(r)  + "*u*cos(v)*" +
                                     Double.toString(sin_lon));
      par_angle_fun.setExpression(2, Double.toString(r) + "*u*sin(v)");

      par_angle_sur.setDomainDescr(par_angle_dom);
      par_angle_sur.setFunctionExpr(par_angle_fun);
      par_angle_sur.compute();
      par_angle_sur.update(par_angle_sur);
   }; // End of updateParallelAngle()

   //////////////////////////////////////////////////////////////////////////
   // Updates meridian angle (longitude) surfaces
   //////////////////////////////////////////////////////////////////////////
   protected void updateMeridianAngle(double lon_in_rad, double r, Color c)
   {
      mer_angle_dom = new PgDomainDescr(2);
      mer_angle_dom.setName("Domain of the Meridian Angle Surface");
      if (lon_in_rad > 0.0)
      {
          mer_angle_dom.setMaxSize(0.0, 0.0, lon_in_rad, globe_radius);
          mer_angle_dom.setSize(0.0, 0.0, lon_in_rad, globe_radius);
      }
      else
      {
          mer_angle_dom.setMaxSize(lon_in_rad, 0.0, 0.0, globe_radius);
          mer_angle_dom.setSize(lon_in_rad, 0.0, 0.0, globe_radius);
      };
      mer_angle_dom.setDiscr(meridian_angle_n_of_p, 2);
      mer_angle_dom.init();

      mer_angle_sur.setDomainDescr(mer_angle_dom);
      mer_angle_sur.compute();
      mer_angle_sur.update(mer_angle_sur);
   }; // End of updateMeridianAngle()

   //////////////////////////////////////////////////////////////////////////
   // Converts a latitude angle (in degrees) to the format X XX' [N|S]
   //////////////////////////////////////////////////////////////////////////
   public String lat2str(double lat_in_deg)
   {
      int l_i = (int) Math.floor(Math.abs(lat_in_deg*100));
      int deg; // Number of degrees
      int min; // Number of minutes
      String ans;

      if (l_i == 0)
      {
         return("0 00'");
      };

      if (l_i <= 99)
      {
         min = (int) Math.floor(l_i*60/100.0);
         deg = 0;
      }
      else
      {
          String l_i_s = Integer.toString(l_i);
          deg = Integer.parseInt(l_i_s.substring(0, l_i_s.length() - 2));
          min = Integer.parseInt(l_i_s.substring(l_i_s.length() - 2,
                                                 l_i_s.length()));
          min = (int) Math.floor(min*60/100.0);
      };

      ans = Integer.toString(deg) + " ";

      if (min <= 9)
      {
         ans = ans + "0" + Integer.toString(min) + "'";
      }
      else
      {
         ans = ans + Integer.toString(min) + "'";
      };

      if (lat_in_deg < 0.0)
      {
         ans = ans + " " + S_trans[language];
      }
      else
      {
         ans = ans + " " + N_trans[language];
      };

      return(ans);
   }; // End of lat2str();

   //////////////////////////////////////////////////////////////////////////
   // Converts a longitude angle (in degrees) to the format X XX' [W|E]
   //////////////////////////////////////////////////////////////////////////
   public String lon2str(double lon_in_deg)
   {
       int l_i = (int) Math.floor(Math.abs(lon_in_deg*100));
       int deg;
       int min;
       String ans;

       if (l_i == 0)
       {
          return("0 00'");
       };

       if (l_i <= 99)
       {
          min = (int) Math.floor(l_i*60/100.0);
          deg = 0;
       }
       else
       {
           String l_i_s = Integer.toString(l_i);
           deg = Integer.parseInt(l_i_s.substring(0, l_i_s.length() - 2));
           min = Integer.parseInt(l_i_s.substring(l_i_s.length() - 2,
                                                  l_i_s.length()));
           min = (int) Math.floor(min*60/100.0);
       };

       ans = Integer.toString(deg) + " ";

       if (min <= 9)
       {
          ans = ans + "0" + Integer.toString(min) + "'";
       }
       else
       {
          ans = ans + Integer.toString(min) + "'";
       };

       if (lon_in_deg < 0.0)
       {
          ans = ans + " " + W_trans[language];
       }
       else
       {
          ans = ans + " " + E_trans[language];
       };

       return(ans);
   }; // End of lon2str()

   //////////////////////////////////////////////////////////////////////////
   // Computes the cartesian coordinates of a point knowing its latitude and
   // longitude.
   //////////////////////////////////////////////////////////////////////////
   protected PdVector computePosition(double lat_in_deg, double lon_in_deg)
   {
       double lat_in_rad = lat_in_deg*Math.PI/180.0;
       double lon_in_rad = lon_in_deg*Math.PI/180.0;
       double r = globe_radius;
       double x = r*Math.cos(lat_in_rad)*Math.cos(lon_in_rad);
       double y = r*Math.cos(lat_in_rad)*Math.sin(lon_in_rad);
       double z = r*Math.sin(lat_in_rad);
       return(new PdVector(x, y, z));
   }; // End of computePosition()

   //////////////////////////////////////////////////////////////////////////
   // Updates picked point position
   //////////////////////////////////////////////////////////////////////////
   protected void setPosition(double lat_in_deg, double lon_in_deg,
		              boolean adjustCamera)
   {
      double lat_in_rad = lat_in_deg*Math.PI/180.0;
      double lon_in_rad = lon_in_deg*Math.PI/180.0;
      double r = globe_radius;

      // Updates parallel through picked point
      for (int i = 0; i <= parallel_n_of_p; i++)
      {
         double lon = -Math.PI + (1.0*i)/parallel_n_of_p*2.0*Math.PI;
         par_by_point.setVertex(i, r*Math.cos(lat_in_rad)*Math.cos(lon),
                                   r*Math.cos(lat_in_rad)*Math.sin(lon),
                                   r*Math.sin(lat_in_rad));
      };

      // Updates meridian through picked point
      for (int i = 0; i <= meridian_n_of_p; i++)
      {
         double lat = -Math.PI/2 + (1.0*i)/meridian_n_of_p*Math.PI;
         mer_by_point.setVertex(i, r*Math.cos(lat) * Math.cos(lon_in_rad),
                                   r*Math.cos(lat) * Math.sin(lon_in_rad),
                                   r*Math.sin(lat));
      };
      par_by_point.update(par_by_point);
      mer_by_point.update(mer_by_point);

      // Updates latitude and longitude labels
      lat_l.setText(lat2str(lat_in_deg));
      lon_l.setText(lon2str(lon_in_deg));

      // Updates picked point position
      double x = r*Math.cos(lat_in_rad)*Math.cos(lon_in_rad);
      double y = r*Math.cos(lat_in_rad)*Math.sin(lon_in_rad);
      double z = r*Math.sin(lat_in_rad);
      picked_point.setVertex(1, x, y, z);
      picked_point.update(picked_point);

      // Updates latitude and longitude angles
      updateParallelAngle(lat_in_rad, lon_in_rad, globe_radius, Color.RED);
      updateMeridianAngle(lon_in_rad, globe_radius, Color.RED);

      ray2.setVertex(1, r*x/Math.sqrt(x*x + y*y), r*y/Math.sqrt(x*x + y*y), 0);
      ray2.update(ray2);

      if (adjustCamera == true)
      {
         // Changes camera position
         PvDisplayIf disp = getDisplay();
         if (disp != null)
         {
	    double dist = 2.5;
            PdVector d1 = new PdVector(dist*x, dist*y, dist*z);
	    PdVector d2 = new PdVector(3);
	    d2.copy(d1);
	    d2.multScalar(-1);
	    PvCameraIf cam = disp.getCamera();
	    cam.setPosition(d1);
	    cam.setViewDir(d2);
	    cam.setRoll(0);
	    cam.update(cam);
         };
      };
   }; // End of setPosition()

   //////////////////////////////////////////////////////////////////////////
   //  Returns a PgElement of our so beloved unit sphere S^2.
   //////////////////////////////////////////////////////////////////////////
   public PgElementSet getSphere(String name, double radius, boolean alpha)
   {
      PgElementSet sphere = new PgElementSet();
      sphere.computeSphere(m_numULines, m_numVLines, radius);
      PgElementSet.triangulate(sphere);
      sphere.makeVertexNormals();
      sphere.makeElementNormals();
      sphere.showBackface(false);
      sphere.showEdges(false);
      sphere.showElements(true);

      sphere.setDimOfTextures(2);
      sphere.assureVertexTextures();
      sphere.showVertexTexture(true);
      sphere.showVertices(false);
      sphere.showElementColors(false);
      sphere.showEdgeColors(false);

      // Generate texture coordinates in [0,1]*[0,1]
      PdVector[] texCoord = sphere.getVertexTextures();
      double uFac = 1. / (-1. + m_numULines);
      double vFac = 1. / (-1. + m_numVLines);
      int ind = 0;
      for (int i = 0; i < m_numULines; i++)
      {
         double u = uFac * i;
	 for (int j = 0; j < m_numVLines; j++)
	 {
	    double v = vFac * j;
	    texCoord[ind].m_data[0] = u;
	    texCoord[ind].m_data[1] = v;
	    ind++;
	 };
      };
      URL url = PjEarth.class.getResource("/earth/flags/earth.jpg");
      PsImage texImage = new PsImage(url);

      if (texImage != null)
      {
	 texImage.loadImage();
	 Image img = texImage.getImage();

         if (img != null)
         {
            PgTexture texture = new PgTexture();
	    texture.setImage(img);
	    if (alpha)
	    {
	       texture.setBlendingMode(PgJvxSrc.BLEND_ALPHA);
	       sphere.setGlobalElementColor(new Color(255, 255, 255));
	    }
	    else
	    {
	       texture.setBlendingMode(PgTexture.BLEND_REPLACE);
	    };
	    sphere.setTexture(texture);
    	    sphere.update(sphere);
         };
      };

      sphere.showTransparency(true);
      sphere.setTransparency(defaultTrans);
      sphere.showSmoothLighting(true);

      sphere.setName(name);
      return(sphere);
   };


   //////////////////////////////////////////////////////////////////////////
   // Draws the geodesical arc joining two cities over the globe
   //////////////////////////////////////////////////////////////////////////
   public double drawSphereGeodesic(PdVector city1_pos, PdVector city2_pos,
                                  String city1_name, String city2_name)
   {
       // The order here is important! You have to set up the point
       // position before you change its name.
       city1_seg.setVertex(1, city1_pos);
       city2_seg.setVertex(1, city2_pos);
       city1_seg.getVertex(1).setName(city1_name);
       city2_seg.getVertex(1).setName(city2_name);
       city1_seg.update(city1_seg);
       city2_seg.update(city2_seg);


       double angle = city1_pos.angle(city1_pos, city2_pos); // In degrees
       if (Math.abs(angle) < 0.2)
       {
          for (int i = 0; i < geodesic_n_of_p; i++)
          {
             geodesic.setVertex(i, city1_pos);
          };
       }
       else
       {

           PdVector u = new PdVector(city1_pos.getEntry(0),
                                     city1_pos.getEntry(1),
                                     city1_pos.getEntry(2));
           u.normalize();
           PdVector w = u.crossNew(u, city2_pos);
           w.normalize();
           PdVector v = w.crossNew(w, u);
           v.normalize();

           double r = 1.01*globe_radius;
           double t;
           for (int i = 0; i < geodesic_n_of_p; i++)
           {
              t = i*angle/(geodesic_n_of_p - 1)*Math.PI/180.0;

              double x = u.getEntry(0)*r*Math.cos(t) +
                         v.getEntry(0)*r*Math.sin(t);
              double y = u.getEntry(1)*r*Math.cos(t) +
                         v.getEntry(1)*r*Math.sin(t);
              double z = u.getEntry(2)*r*Math.cos(t) +
                         v.getEntry(2)*r*Math.sin(t);

              geodesic.setVertex(i, x, y, z);
           };
       };
       geodesic.update(geodesic);

       if (show_geodesic == false)
       {
          addGeometry(city1_seg);
          addGeometry(city2_seg);
          addGeometry(geodesic);

          show_geodesic = true;
       };

       return(2.0*Math.PI*earth_radius*angle/360.0);
   };

  /***************************************************************************
   ***************************************************************************
   **  Begin of PvPickListenerIf Implementation
   ***************************************************************************
   ***************************************************************************/

   //////////////////////////////////////////////////////////////////////////
   // The name of a listeners allows the display to issue verbal debug
   // messages.
   //////////////////////////////////////////////////////////////////////////
   public String getName()
   {
      return("EarthPicker");
   }; // End of getName()

   //////////////////////////////////////////////////////////////////////////
   // Get a location in the display with 2d display and 3d world coordinates.
   // Method is called when display is in mode PvDisplayIf.MODE_DISPLAY_PICK.
   //////////////////////////////////////////////////////////////////////////
   public void pickDisplay(PvPickEvent pos)
   {
      // PsDebug.message("pickDisplay() called ...");
      // System.out.println("pickDisplay() called ...");
   }; // End of pickDisplay()

   //////////////////////////////////////////////////////////////////////////
   // Drag a location in the display with 2d display and 3d world coordinates.
   // Method is called when display is in mode PvDisplayIf.MODE_DISPLAY_PICK.
   //////////////////////////////////////////////////////////////////////////
   public void dragDisplay(PvPickEvent pos)
   {
      // PsDebug.message("pickDisplay() called ...");
      // System.out.println("pickDisplay() called ...");
   }; // End of dragDisplay()

   //////////////////////////////////////////////////////////////////////////
   // Pick an arbitrary point on a geometry, point may lie inside an element.
   // Method is called when display is in mode PvDisplayIf.MODE_INITIAL_PICK
   // or if temporarily the i-key is pressed, and any pixel in the display is
   // picked.
   //////////////////////////////////////////////////////////////////////////
   public void pickInitial(PvPickEvent pos)
   {
      // PsDebug.message("pickInitial() called ...");
      // System.out.println("pickInitial() called ...");
      double x = pos.getVertex().getEntry(0);
      double y = pos.getVertex().getEntry(1);
      double z = pos.getVertex().getEntry(2);

      PdVector sph = pos2geo(x, y, z);

      double lat_in_deg = sph.getEntry(1);
      double lon_in_deg = sph.getEntry(0);

      setPosition(lat_in_deg, lon_in_deg, false);

      picked_point.getVertex(0).setName("");
      picked_point.getVertex(1).setName("");
      picked_point.update(picked_point);

      if (show_geodesic == true)
      {
        removeGeometry(city1_seg);
        removeGeometry(city2_seg);
        removeGeometry(geodesic);
        show_geodesic = false;
	distLabel.setText(ndef_trans);
      };
   }; // End of pickInitial()

   //////////////////////////////////////////////////////////////////////////
   // Drag an arbitrary point along a geometry, point may lie inside an
   // element. Method is called when display is in mode
   // PvDisplayIf.MODE_INITIAL_PICK or if temporarily the i-key is pressed,
   // and any pixel in the display is dragged.
   //////////////////////////////////////////////////////////////////////////
   public void dragInitial(PvPickEvent pos)
   {
      // PsDebug.message("dragInitial() called ...");
      // System.out.println("dragInitial() called ...");
      double x = pos.getVertex().getEntry(0);
      double y = pos.getVertex().getEntry(1);
      double z = pos.getVertex().getEntry(2);

      PdVector sph = pos2geo(x, y, z);

      double lat_in_deg = sph.getEntry(1);
      double lon_in_deg = sph.getEntry(0);

      setPosition(lat_in_deg, lon_in_deg, false);

      picked_point.getVertex(0).setName("");
      picked_point.getVertex(1).setName("");
      picked_point.update(picked_point);
   }; // End of dragInitial()

   //////////////////////////////////////////////////////////////////////////
   // Get a picked vertex of a geometry.
   // Method is called when display is in mode PvDisplayIf.MODE_PICK
   // or if temporarily the p-key is pressed, and a vertex picked.
   //////////////////////////////////////////////////////////////////////////
   public void pickVertex(PgGeometryIf geom, int index, PdVector vertex)
   {
       // PsDebug.message("pickVertex() called ...");
       // System.out.println("pickVertex() called ...");
   }; // End of dragInitial();

   //////////////////////////////////////////////////////////////////////////
   // Method is called when display is in mode PvDisplayIf.MODE_PICK
   // or if temporarily the p-key is pressed, and a vertex dragged.
   //////////////////////////////////////////////////////////////////////////
   public void dragVertex(PgGeometryIf a_geom, int index, PdVector vertex)
   {
      // PsDebug.message("dragVertex() called ...");
      // System.out.println("dragVertex() called ...");
   }; // End of dragVertex()

   //////////////////////////////////////////////////////////////////////////
   // Mark a set of vertices of a geometry within a given bounding box.
   // Method is called when display is in mode PvDisplayIf.MODE_MARK
   // or if temporarily the m-key is pressed, and a rectangle is drawn.
   //////////////////////////////////////////////////////////////////////////
   public void markVertices(PvPickEvent markBox)
   {
      // PsDebug.message("markVertices() called ...");
      // System.out.println("markVertices() called ...");
   }; // End of markVertices()

   //////////////////////////////////////////////////////////////////////////
   // Unmark a set of vertices of a geometry within a given bounding box.
   // Method is called when display is in mode PvDisplayIf.MODE_UNMARK
   // or if temporarily the u-key is pressed, and a rectangle is drawn.
   //////////////////////////////////////////////////////////////////////////
   public void unmarkVertices(PvPickEvent markBox)
   {
      // PsDebug.message("unmarkVertices() called ...");
      // System.out.println("unmarkVertices() called ...");
   }; // End of unmarkVertices()

  /***************************************************************************
   ***************************************************************************
   **  Begin of PvPickListenerIf Implementation
   ***************************************************************************
   ***************************************************************************/
}; // End of PjEarth classs


