///////////////////////////////////////////////////////////////////////////////
// Earth Interface class
//
// Links: http://pt.wikipedia.org/wiki/Lista_de_pa%C3%ADses
//        http://en.wikipedia.org/wiki/List_of_countries
//        http://geom.mi.fu-berlin.de/hurrelmann/
//
// Authors: R. V. de Almeira Jr, J. Hurrelmann, K. Polthier and
//          H. J. Bortolossi.
//
// Created: 06/12/2005 -- Initial design and implementation.
// Updated: 10/05/2006 -- Earth's texture file added.
//
// Last update: 10/05/2006
///////////////////////////////////////////////////////////////////////////////
package earth;

///////////////////////////////////////////////////////////////////////////////
// Native JAVA imports
///////////////////////////////////////////////////////////////////////////////
import java.applet.AppletContext;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Checkbox;
import java.awt.Choice;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Label;
import java.awt.Panel;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import java.net.URL;

import jv.vecmath.PdVector;
import jv.objectGui.PsTabPanel;


///////////////////////////////////////////////////////////////////////////////
// JavaView imports
///////////////////////////////////////////////////////////////////////////////
import jv.object.PsPanel;
import jv.object.PsUpdateIf;
import jv.project.PjProject_IP;

///////////////////////////////////////////////////////////////////////////////
// Info panel for graph tomography applet.
///////////////////////////////////////////////////////////////////////////////
public class PjEarth_IP extends PjProject_IP implements ItemListener, ActionListener
{
   protected PjEarth m_pjEarth;

   protected Label titleLabel;

   protected Checkbox coastline_cb;
   protected Checkbox globe_cb;
   protected Checkbox equator_cb;
   protected Checkbox greenwich_cb;
   protected Checkbox cancer_cb;
   protected Checkbox capricorn_cb;
   protected Checkbox arctic_circle_cb;
   protected Checkbox antarctic_circle_cb;
   protected Checkbox mer_angle_sur_cb;
   protected Checkbox par_angle_sur_cb;
   protected Checkbox earth_sur_cb;
   protected Checkbox america_bnd_cb;
   protected Checkbox asia_bnd_cb;
   protected Checkbox europe_bnd_cb;
   protected Checkbox africa_bnd_cb;
   protected Checkbox oceania_bnd_cb;

   protected PsTabPanel tabPanel;

   protected PsPanel latPanel;
   protected PsPanel lonPanel;
   protected PsPanel posPanel;

   protected PsPanel locPanel;
   protected Choice locChoice;
   protected Label capLabel;
   protected DrawingPanel flagPanel;
   protected Button encButton;

   protected int language;
   protected String[] title_trans = {"Paralelos e Meridianos da Terra",
                                     "Parallels and Meridians of the Earth"};
   protected String[] and_trans =   {"e", "and"};

   protected String[]
      coastline_trans = {"Linha Costeira", "Coastline"};
   protected String[]
      globe_trans = {"Globo Terrestre", "Earth's Globe"};
   protected String[]
      equator_trans = {"Equador", "Equator"};
   protected String[]
      greenwich_trans = {"Greenwich", "Meridian"};
   protected String[]
      cancer_trans = {"Trpico de Cncer", "Tropic of Cancer"};
   protected String[]
      capricorn_trans = {"Trpico de Capricrnio", "Tropic of Capricorn"};
   protected String[]
      arctic_circle_trans = {"Crculo Polar rtico", "Arctic Circle"};
   protected String[]
      antarctic_circle_trans = {"Crculo Polar Antrtico", "Antarctic Circle"};
   protected String[]
      par_angle_sur_trans = {"ngulo do Paralelo", "Parallel's Angle"};
   protected String[]
      mer_angle_sur_trans = {"ngulo do Meridiano", "Meridian's Angle"};

   protected String[]
      earth_sur_trans = {"Superfcie da Terra", "Earth's Surface"};

   protected String
      bnd_trans[] = {"Fronteiras", "Boundaries"};
   protected String
      america_bnd_trans[] = {"Amrica", "America"};
   protected String
      asia_bnd_trans[] = {"sia", "Asia"};
   protected String[]
      europe_bnd_trans = {"Europa", "Europe"};
   protected String[]
      africa_bnd_trans = {"frica", "Africa"};
   protected String[]
      oceania_bnd_trans = {"Oceania", "Oceania"};

   protected String[]
      vis_trans = {"Visualizao", "Visualization"};
   protected String[]
      pos_trans = {"Posio", "Position"};
   protected String[]
      loc_trans = {"Lugar", "Place"};
   protected String[]
      msg_trans = {"Escolha um lugar ...", "Choose a place ..."};
   protected String[]
      enc_trans = {"Consultar Wikipdia", "Consult Wikipedia"};
   protected String[]
      def_trans = {"http://pt.wikipedia.org/wiki/Lista_de_pa%C3%ADses",
                   "http://en.wikipedia.org/wiki/List_of_countries"};
   protected String encURL;
   protected String[] lat_trans = {"Latitude", "Latitude"};
   protected String[] lon_trans = {"Longitude", "Longitude"};

   protected int number_of_cities = 196;
   protected Location[] loc = new Location[number_of_cities];

   protected int[] capital_sorted = new int[number_of_cities];
   protected String[] dist_trans = {"A distncia  aproximadamente", "The distance is approximately"};
   protected String[] ndef_trans = {"Distncia: escolha duas cidades!", "Distance: choose two cities!"};
   protected Choice city1Choice;
   protected Choice city2Choice;
   protected int city1_idx = -1;
   protected int city2_idx = -1;
   protected PdVector city1_vec;
   protected PdVector city2_vec;
   protected String city1_name;
   protected String city2_name;


   ///////////////////////////////////////////////////////////////////////////
   // Default constructor
   ///////////////////////////////////////////////////////////////////////////
   public PjEarth_IP()
   {
      // Calls parent
      super();

      if (getClass() == PjEarth_IP.class)
      {
         init();
      };

/*
      for (int i = 0; i < number_of_cities; i++)
      {
         capital_sorted[i] = i;
      };
 */
   }; // End of PjEarth_IP()


   //////////////////////////////////////////////////////////////////////////
   // Setup the main control panel
   //////////////////////////////////////////////////////////////////////////
   public void init()
   {
      super.init();

      // Title
      addTitle(title_trans[language]);

      // Info
      Panel m_pInfo =  new Panel();
      m_pInfo.setLayout(new FlowLayout(FlowLayout.RIGHT));
      add(m_pInfo);
      titleLabel = new Label("R. V. de Almeira Jr, " +
                             "J. Hurrelmann, " +
                             "K. Polthier " +
                             and_trans[language] +
                             " H. J. Bortolossi");
      titleLabel.setFont(new Font("TimesRoman", Font.ITALIC, 10));
      m_pInfo.add(titleLabel);

      tabPanel = new PsTabPanel();
      add(tabPanel);
   }; // End of init()

   //////////////////////////////////////////////////////////////////////////
   // Setup parent
   //////////////////////////////////////////////////////////////////////////
     public void setParent(PsUpdateIf parent)
   {
       // Gets pointer to the main project
       super.setParent(parent);
       m_pjEarth = (PjEarth) parent;
       language = m_pjEarth.language;

       // Setup title
       setTitle(title_trans[language]);
       titleLabel.setText("R. V. de Almeira Jr, " +
                          "J. Hurrelmann, " +
                          "K. Polthier " +
                          and_trans[language] +
                          " H. J. Bortolossi"
);

       // Setup visualization tab
       PsPanel visPanel = new PsPanel();
       visPanel.setLayout(new GridLayout(12, 1));
       visPanel.add(new Label(" "));

       // Earth's surface
       earth_sur_cb = new Checkbox(earth_sur_trans[language]);
       earth_sur_cb.addItemListener(this);
       earth_sur_cb.setState(m_pjEarth.viewEarthSurface);
       visPanel.add(earth_sur_cb);

       PsPanel optPanel1 = new PsPanel();
       PsPanel optPanel2 = new PsPanel();
       PsPanel optPanel3 = new PsPanel();
       PsPanel optPanel4 = new PsPanel();
       PsPanel optPanel5 = new PsPanel();
       optPanel1.setLayout(new GridLayout(1, 2));
       optPanel2.setLayout(new GridLayout(1, 2));
       optPanel3.setLayout(new GridLayout(1, 2));
       optPanel4.setLayout(new GridLayout(1, 2));
       optPanel5.setLayout(new GridLayout(1, 2));
       // Coastline
       coastline_cb = new Checkbox(coastline_trans[language]);
       coastline_cb.addItemListener(this);
       coastline_cb.setState(m_pjEarth.viewCoastline);
       optPanel1.add(coastline_cb);
       // Globe
       globe_cb = new Checkbox(globe_trans[language]);
       globe_cb.addItemListener(this);
       globe_cb.setState(m_pjEarth.viewGlobe);
       optPanel1.add(globe_cb);
       // Equator
       equator_cb = new Checkbox(equator_trans[language]);
       equator_cb.addItemListener(this);
       equator_cb.setState(m_pjEarth.viewEquator);
       optPanel2.add(equator_cb);
       // Greenwich
       greenwich_cb = new Checkbox(greenwich_trans[language]);
       greenwich_cb.addItemListener(this);
       greenwich_cb.setState(m_pjEarth.viewGreenwich);
       optPanel2.add(greenwich_cb);
       // Cancer
       cancer_cb = new Checkbox(cancer_trans[language]);
       cancer_cb.addItemListener(this);
       cancer_cb.setState(m_pjEarth.viewCancer);
       optPanel3.add(cancer_cb);
       // Capricorn
       capricorn_cb = new Checkbox(capricorn_trans[language]);
       capricorn_cb.addItemListener(this);
       capricorn_cb.setState(m_pjEarth.viewCapricorn);
       optPanel3.add(capricorn_cb);
       // Artic
       arctic_circle_cb = new Checkbox(arctic_circle_trans[language]);
       arctic_circle_cb.addItemListener(this);
       arctic_circle_cb.setState(m_pjEarth.viewArctic);
       optPanel4.add(arctic_circle_cb);
       // Antartic
       antarctic_circle_cb = new Checkbox(antarctic_circle_trans[language]);
       antarctic_circle_cb.addItemListener(this);
       antarctic_circle_cb.setState(m_pjEarth.viewAntarctic);
       optPanel4.add(antarctic_circle_cb);
       // Latitude angle
       par_angle_sur_cb = new Checkbox(par_angle_sur_trans[language]);
       par_angle_sur_cb.addItemListener(this);
       par_angle_sur_cb.setState(m_pjEarth.viewParalellAngle);
       optPanel5.add(par_angle_sur_cb);
       // Longitude angle
       mer_angle_sur_cb = new Checkbox(mer_angle_sur_trans[language]);
       mer_angle_sur_cb.addItemListener(this);
       mer_angle_sur_cb.setState(m_pjEarth.viewMeridianAngle);
       optPanel5.add(mer_angle_sur_cb);
       visPanel.add(optPanel1);
       visPanel.add(optPanel2);
       visPanel.add(optPanel3);
       visPanel.add(optPanel4);
       visPanel.add(optPanel5);
       visPanel.add(new Label(" "));
       visPanel.add(new Label(bnd_trans[language]));

       PsPanel optPanel6 = new PsPanel();
       PsPanel optPanel7 = new PsPanel();
       optPanel6.setLayout(new GridLayout(1, 2));
       optPanel7.setLayout(new GridLayout(1, 2));
       // Americas boundaries
       america_bnd_cb = new Checkbox(america_bnd_trans[language]);
       america_bnd_cb.addItemListener(this);
       america_bnd_cb.setState(m_pjEarth.viewAmericaBnd);
       optPanel6.add(america_bnd_cb);
       // Asia boundaries
       asia_bnd_cb = new Checkbox(asia_bnd_trans[language]);
       asia_bnd_cb.addItemListener(this);
       asia_bnd_cb.setState(m_pjEarth.viewAsiaBnd);
       optPanel6.add(asia_bnd_cb);
       visPanel.add(optPanel6);
       // Europe
       europe_bnd_cb = new Checkbox(europe_bnd_trans[language]);
       europe_bnd_cb.addItemListener(this);
       europe_bnd_cb.setState(m_pjEarth.viewEuropeBnd);
       optPanel7.add(europe_bnd_cb);
       // Africa
       africa_bnd_cb = new Checkbox(africa_bnd_trans[language]);
       africa_bnd_cb.addItemListener(this);
       africa_bnd_cb.setState(m_pjEarth.viewAfricaBnd);
       optPanel7.add(africa_bnd_cb);
       visPanel.add(optPanel7);
       // Oceania
       oceania_bnd_cb = new Checkbox(oceania_bnd_trans[language]);
       oceania_bnd_cb.addItemListener(this);
       oceania_bnd_cb.setState(m_pjEarth.viewOceaniaBnd);
       visPanel.add(oceania_bnd_cb);
       tabPanel.addPanel(vis_trans[language], visPanel);

       // Setup position tab
       posPanel = new PsPanel();
       tabPanel.addPanel(pos_trans[language], posPanel);

       latPanel = new PsPanel();
       lonPanel = new PsPanel();

       latPanel.setLayout(new GridLayout(1, 2));
       latPanel.removeAll();
       latPanel.add(new Label(lat_trans[language] + ":"));
       latPanel.add(m_pjEarth.lat_l);

       lonPanel.setLayout(new GridLayout(1, 2));
       lonPanel.removeAll();
       lonPanel.add(new Label(lon_trans[language] + ":"));
       lonPanel.add(m_pjEarth.lon_l);

       posPanel.add(new Label(" "));
       posPanel.add(latPanel);
       posPanel.add(lonPanel);

       // Setup location tab
       locPanel = new PsPanel();
       // locPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
       locChoice = new Choice();
       locPanel.add(new Label(""));
       locPanel.add(locChoice);
       locPanel.add(new Label(""));
       encButton = new Button(enc_trans[language]);
       locPanel.add(encButton);
       encButton.addActionListener(this);
       encURL = def_trans[language];
       capLabel = new Label(" ", Label.CENTER);
       locPanel.add(capLabel);
       locPanel.add(new Label(" "));
       flagPanel = new DrawingPanel();
       locPanel.add(flagPanel);

       locChoice.addItemListener(this);
       locChoice.add(msg_trans[language]);
       buildPlaces();

       tabPanel.addPanel(loc_trans[language], locPanel);

       switch(m_pjEarth.tabActive)
       {
          case 2:
              tabPanel.setVisible(pos_trans[language]);
          break;

          case 3:
              tabPanel.setVisible(loc_trans[language]);
          break;

          default:
              tabPanel.setVisible(vis_trans[language]);
       };

       tabPanel.validate();

       // Setup initial location if any
       if (m_pjEarth.initialLocation.equals("none") == false)
       {
         for(int i = 0; i < number_of_cities; i++)
         {
            if (m_pjEarth.initialLocation.equals(loc[i].getCountryName(1))
			                                              == true)
            {
               m_pjEarth.setPosition(loc[i].getLatitude(),
                                     loc[i].getLongitude(), true);
               m_pjEarth.picked_point.getVertex(1).setName(loc[i].getCapital(language) + "  (" + loc[i].getCountryName(language) + ")");
               m_pjEarth.picked_point.update(m_pjEarth.picked_point);
               capLabel.setText("Capital: " + loc[i].getCapital(language));
               encURL = loc[i].getWiki(language);

               String flag = loc[i].getFlag();
               if (!(flag.equals("none")))
               {
//                 URL url = PjEarth_IP.class.getResource("/flags/" + loc[i].getFlag());
//                 URL url = PjEarth_IP.class.getResource("flags/" + loc[i].getFlag());
                   URL url = PjEarth_IP.class.getResource("/earth/flags/" + loc[i].getFlag());
                   Image img = Toolkit.getDefaultToolkit().getImage(url);
                   while (img.getHeight(this) == -1)
                   {
                      // Empty!
                      // This is set to wait complete load of the image!
                   };
                   flagPanel.setImage(img, locPanel.getWidth());
                   flagPanel.update(flagPanel.getGraphics());
               };

               String continent = loc[i].getContinent();
               if (america_bnd_cb.getState())
               {
                   m_pjEarth.removeGeometry(m_pjEarth.america_bnd);
                   america_bnd_cb.setState(false);
               };
               if (asia_bnd_cb.getState())
               {
                   m_pjEarth.removeGeometry(m_pjEarth.asia_bnd);
                   asia_bnd_cb.setState(false);
               };
               if (africa_bnd_cb.getState())
               {
                   m_pjEarth.removeGeometry(m_pjEarth.africa_bnd);
                   africa_bnd_cb.setState(false);
               };
               if (europe_bnd_cb.getState())
               {
                   m_pjEarth.removeGeometry(m_pjEarth.europe_bnd);
                   europe_bnd_cb.setState(false);
               };
               if (oceania_bnd_cb.getState())
               {
                   m_pjEarth.removeGeometry(m_pjEarth.oceania_bnd);
                   oceania_bnd_cb.setState(false);
               };
               if (continent.equals("america"))
               {
                   america_bnd_cb.setState(true);
                   m_pjEarth.addGeometry(m_pjEarth.america_bnd);
               };
               if (continent.equals("asia"))
               {
                   asia_bnd_cb.setState(true);
                   m_pjEarth.addGeometry(m_pjEarth.asia_bnd);
               };
               if (continent.equals("africa"))
               {
                   africa_bnd_cb.setState(true);
                   m_pjEarth.addGeometry(m_pjEarth.africa_bnd);
               };
               if (continent.equals("europe"))
               {
                   europe_bnd_cb.setState(true);
                   m_pjEarth.addGeometry(m_pjEarth.europe_bnd);
               };
               if (continent.equals("oceania"))
               {
                   oceania_bnd_cb.setState(true);
                   m_pjEarth.addGeometry(m_pjEarth.oceania_bnd);
               };
            };
          };
       };

       // Update position panel
       posPanel.add(new Label(" "));
       posPanel.addLine(1);
       posPanel.add(new Label(" "));
       m_pjEarth.ndef_trans = ndef_trans[language];
       m_pjEarth.distLabel.setText(ndef_trans[language]);
       posPanel.add(m_pjEarth.distLabel);
       city1Choice = new Choice();
       city1Choice.addItemListener(this);
       city2Choice = new Choice();
       city2Choice.addItemListener(this);
       posPanel.add(city1Choice);
       city1Choice.add(msg_trans[language]);
       posPanel.add(city2Choice);
       city2Choice.add(msg_trans[language]);
       for (int i = 0; i < number_of_cities; i++)
       {
          city1Choice.add(loc[capital_sorted[i]].getCapital(language) +
                          " (" +
                          loc[capital_sorted[i]].getCountryName(language) +
                          ")");
          city2Choice.add(loc[capital_sorted[i]].getCapital(language) +
                          " (" +
                          loc[capital_sorted[i]].getCountryName(language) +
                          ")");
       };
       super.setParent(parent);
   }; // End of setParent()

   //////////////////////////////////////////////////////////////////////////
   // Update the panel whenever the parent has changed somewhere else
   //////////////////////////////////////////////////////////////////////////
   public boolean update(Object  event)
   {
      if (m_pjEarth == event)
      {
         // Fill here!
         return(true);
      };

      return(super.update(event));
   }; // End of update()

   //////////////////////////////////////////////////////////////////////////
   // Handle action events invoked from itens
   //////////////////////////////////////////////////////////////////////////
   public void itemStateChanged(ItemEvent event)
   {
      if (m_pjEarth == null)
      {
         return;
      };

      Object source = event.getSource();

      if (source == locChoice)
      {
         String city = locChoice.getSelectedItem();

         for (int i = 0; i < number_of_cities; i++)
         {
            if (city.equals(loc[i].getCountryName(language)) == true)
            {
               m_pjEarth.setPosition(loc[i].getLatitude(),
                                     loc[i].getLongitude(), true);
               m_pjEarth.picked_point.getVertex(1).setName(loc[i].getCapital(language) + "  (" + loc[i].getCountryName(language) + ")");
               m_pjEarth.picked_point.update(m_pjEarth.picked_point);
               capLabel.setText("Capital: " + loc[i].getCapital(language));
               encURL = loc[i].getWiki(language);

               String flag = loc[i].getFlag();
               if (!(flag.equals("none")))
               {
//                 URL url = PjEarth_IP.class.getResource("/flags/" + loc[i].getFlag());
//                 URL url = PjEarth_IP.class.getResource("flags/" + loc[i].getFlag());
                   URL url = PjEarth_IP.class.getResource("/earth/flags/" + loc[i].getFlag());
                   Image img = Toolkit.getDefaultToolkit().getImage(url);
                   while (img.getHeight(this) == -1)
                   {
                      // Empty!
                      // This is set to wait complete load of the image!
                   };
                   flagPanel.setImage(img, locPanel.getWidth());
                   flagPanel.update(flagPanel.getGraphics());
               };

               String continent = loc[i].getContinent();
               if (america_bnd_cb.getState())
               {
                   m_pjEarth.removeGeometry(m_pjEarth.america_bnd);
                   america_bnd_cb.setState(false);
               };
               if (asia_bnd_cb.getState())
               {
                   m_pjEarth.removeGeometry(m_pjEarth.asia_bnd);
                   asia_bnd_cb.setState(false);
               };
               if (africa_bnd_cb.getState())
               {
                   m_pjEarth.removeGeometry(m_pjEarth.africa_bnd);
                   africa_bnd_cb.setState(false);
               };
               if (europe_bnd_cb.getState())
               {
                   m_pjEarth.removeGeometry(m_pjEarth.europe_bnd);
                   europe_bnd_cb.setState(false);
               };
               if (oceania_bnd_cb.getState())
               {
                   m_pjEarth.removeGeometry(m_pjEarth.oceania_bnd);
                   oceania_bnd_cb.setState(false);
               };
               if (continent.equals("america"))
               {
                   america_bnd_cb.setState(true);
                   m_pjEarth.addGeometry(m_pjEarth.america_bnd);
               };
               if (continent.equals("asia"))
               {
                   asia_bnd_cb.setState(true);
                   m_pjEarth.addGeometry(m_pjEarth.asia_bnd);
               };
               if (continent.equals("africa"))
               {
                   africa_bnd_cb.setState(true);
                   m_pjEarth.addGeometry(m_pjEarth.africa_bnd);
               };
               if (continent.equals("europe"))
               {
                   europe_bnd_cb.setState(true);
                   m_pjEarth.addGeometry(m_pjEarth.europe_bnd);
               };
               if (continent.equals("oceania"))
               {
                   oceania_bnd_cb.setState(true);
                   m_pjEarth.addGeometry(m_pjEarth.oceania_bnd);
               };

               break;
            };
         };
      };

      if (source == city1Choice)
      {
         String city1 = city1Choice.getSelectedItem();

         for (int i = 0; i < number_of_cities; i++)
         {
             if (city1.equals(loc[i].getCapital(language) +  " (" +
                              loc[i].getCountryName(language) +
                              ")") == true)
            {
               city1_idx = i;
               city1_vec = m_pjEarth.computePosition(loc[i].getLatitude(),
                                                     loc[i].getLongitude());
               city1_name = loc[i].getCapital(language) + "(" +
                            loc[i].getCountryName(language) + ")";
               break;
            };

            city1_idx = -1;
         };

         if ((city1_idx != -1) && (city2_idx != -1))
         {
             double d;

             d = m_pjEarth.drawSphereGeodesic(city1_vec,  city2_vec,
                                              city1_name, city2_name);
             d = Math.floor(100*d)/100.0;

             m_pjEarth.distLabel.setText(dist_trans[language] + " " +
                                         Double.toString(d) + " km.");
         }
         else
         {
            if (m_pjEarth.show_geodesic == true)
            {
               m_pjEarth.distLabel.setText(ndef_trans[language]);
               m_pjEarth.removeGeometry(m_pjEarth.city1_seg);
               m_pjEarth.removeGeometry(m_pjEarth.city2_seg);
               m_pjEarth.removeGeometry(m_pjEarth.geodesic);
               m_pjEarth.show_geodesic = false;
            };
         };
      };

      if (source == city2Choice)
      {
         String city2 = city2Choice.getSelectedItem();

         for (int i = 0; i < number_of_cities; i++)
         {
            if (city2.equals(loc[i].getCapital(language) +  " (" +
                             loc[i].getCountryName(language) +
                             ")") == true)
            {
               city2_idx = i;
               city2_vec = m_pjEarth.computePosition(loc[i].getLatitude(),
                                                     loc[i].getLongitude());
               city2_name = loc[i].getCapital(language) + "(" +
                            loc[i].getCountryName(language) + ")";
               break;
            };

            city2_idx = -1;
         };

         if ((city1_idx != -1) && (city2_idx != -1))
         {
             double d;
             d = m_pjEarth.drawSphereGeodesic(city1_vec,  city2_vec,
                                              city1_name, city2_name);
             d = Math.floor(100*d)/100.0;

             m_pjEarth.distLabel.setText(dist_trans[language] + " " +
                                         Double.toString(d) + " km.");
         }
         else
         {
            if (m_pjEarth.show_geodesic == true)
            {
               m_pjEarth.distLabel.setText(ndef_trans[language]);
               m_pjEarth.removeGeometry(m_pjEarth.city1_seg);
               m_pjEarth.removeGeometry(m_pjEarth.city2_seg);
               m_pjEarth.removeGeometry(m_pjEarth.geodesic);
               m_pjEarth.show_geodesic = false;
            };
         };
      };


      if (source == coastline_cb)
      {
         if (coastline_cb.getState() == true)
         {
            m_pjEarth.addGeometry(m_pjEarth.coastline);
         }
         else
         {
            m_pjEarth.removeGeometry(m_pjEarth.coastline);
         };
         return;
      };

      if (source == globe_cb)
      {
         if (globe_cb.getState() == true)
         {
            m_pjEarth.globe.setTransparency(0.5);
            m_pjEarth.globe.update(m_pjEarth.globe);
            if (earth_sur_cb.getState() == true)
            {
               earth_sur_cb.setState(false);
               m_pjEarth.removeGeometry(m_pjEarth.earth_sur);
            };
         }
         else
         {
            m_pjEarth.globe.setTransparency(1.0);
            m_pjEarth.globe.update(m_pjEarth.globe);
         };
         return;
      };

      if (source == earth_sur_cb)
      {
         if (earth_sur_cb.getState() == true)
         {
            m_pjEarth.addGeometry(m_pjEarth.earth_sur);
            if (globe_cb.getState() == true)
            {
               globe_cb.setState(false);
               m_pjEarth.globe.setTransparency(1.0);
               m_pjEarth.globe.update(m_pjEarth.globe);
            };
         }
         else
         {
            m_pjEarth.removeGeometry(m_pjEarth.earth_sur);
         };
         return;
      };

      if (source == equator_cb)
      {
         if (equator_cb.getState() == true)
         {
            m_pjEarth.addGeometry(m_pjEarth.equator);
         }
         else
         {
            m_pjEarth.removeGeometry(m_pjEarth.equator);
         };
         return;
      };

      if (source == greenwich_cb)
      {
         if (greenwich_cb.getState() == true)
         {
            m_pjEarth.addGeometry(m_pjEarth.greenwich);
         }
         else
         {
            m_pjEarth.removeGeometry(m_pjEarth.greenwich);
         };
         return;
      };

      if (source == cancer_cb)
      {
         if (cancer_cb.getState() == true)
         {
            m_pjEarth.addGeometry(m_pjEarth.cancer);
         }
         else
         {
            m_pjEarth.removeGeometry(m_pjEarth.cancer);
         };
         return;
      };

      if (source == capricorn_cb)
      {
         if (capricorn_cb.getState() == true)
         {
            m_pjEarth.addGeometry(m_pjEarth.capricorn);
         }
         else
         {
            m_pjEarth.removeGeometry(m_pjEarth.capricorn);
         };
         return;
      };

      if (source == arctic_circle_cb)
      {
         if (arctic_circle_cb.getState() == true)
         {
            m_pjEarth.addGeometry(m_pjEarth.arctic_circle);
         }
         else
         {
            m_pjEarth.removeGeometry(m_pjEarth.arctic_circle);
         };
         return;
      };

      if (source == antarctic_circle_cb)
      {
         if (antarctic_circle_cb.getState() == true)
         {
            m_pjEarth.addGeometry(m_pjEarth.antarctic_circle);
         }
         else
         {
            m_pjEarth.removeGeometry(m_pjEarth.antarctic_circle);
         };
         return;
      };

      if (source == mer_angle_sur_cb)
      {
         if (mer_angle_sur_cb.getState() == true)
         {
            m_pjEarth.addGeometry(m_pjEarth.mer_angle_sur);
            m_pjEarth.addGeometry(m_pjEarth.ray1);
         }
         else
         {
            m_pjEarth.removeGeometry(m_pjEarth.mer_angle_sur);
            m_pjEarth.removeGeometry(m_pjEarth.ray1);
         };
         return;
      };

      if (source == par_angle_sur_cb)
      {
         if (par_angle_sur_cb.getState() == true)
         {
            m_pjEarth.addGeometry(m_pjEarth.par_angle_sur);
            m_pjEarth.addGeometry(m_pjEarth.ray2);
         }
         else
         {
            m_pjEarth.removeGeometry(m_pjEarth.par_angle_sur);
            m_pjEarth.removeGeometry(m_pjEarth.ray2);
         };
         return;
      };

      if(source == america_bnd_cb)
      {
          if(america_bnd_cb.getState())
          {
             m_pjEarth.addGeometry(m_pjEarth.america_bnd);
          }
          else
          {
              m_pjEarth.removeGeometry(m_pjEarth.america_bnd);
          };

          return;
      };

      if(source == asia_bnd_cb)
      {
          if(asia_bnd_cb.getState())
          {
             m_pjEarth.addGeometry(m_pjEarth.asia_bnd);
          }
          else
          {
             m_pjEarth.removeGeometry(m_pjEarth.asia_bnd);
          };
          return;
      };

      if (source == europe_bnd_cb)
      {
          if (europe_bnd_cb.getState() == true)
          {
             m_pjEarth.addGeometry(m_pjEarth.europe_bnd);
          }
          else
          {
             m_pjEarth.removeGeometry(m_pjEarth.europe_bnd);
          };
          return;
      };

      if (source == oceania_bnd_cb)
      {
          if (oceania_bnd_cb.getState() == true)
          {
            m_pjEarth.addGeometry(m_pjEarth.oceania_bnd);
          }
          else
          {
            m_pjEarth.removeGeometry(m_pjEarth.oceania_bnd);
          };
          return;
       };

       if (source == africa_bnd_cb)
       {
          if (africa_bnd_cb.getState() == true)
          {
             m_pjEarth.addGeometry(m_pjEarth.africa_bnd);
          }
          else
          {
             m_pjEarth.removeGeometry(m_pjEarth.africa_bnd);
          };
          return;
       };
   }; // End of itemStateChanged();

   //////////////////////////////////////////////////////////////////////////
   // Handle action events invoked from text fields
   //////////////////////////////////////////////////////////////////////////
   public void actionPerformed(ActionEvent event)
   {
     if (m_pjEarth == null)
     {
         return;
     };

     Object source = event.getSource();
     if (source == encButton)
     {
         java.applet.AppletContext ap = m_pjEarth.appletContext;
         try
         {
            if (ap!=null)
            {
                ap.showDocument(new java.net.URL(encURL), "_blank");
            };
         }
         catch(java.net.MalformedURLException exp)
         {
           System.out.println(exp.toString());
         };
     };
   }; // End of actionPerformed()

   //////////////////////////////////////////////////////////////////////////
   // Build places
   //////////////////////////////////////////////////////////////////////////
   protected void buildPlaces()
   {
       int i = 0;

       loc[i] = new Location(new String[] {"Acre (Brasil)", "Acre (Brazil)"},
                             new String[] {"Rio Branco", "Rio Branco"},
                             -9, -58, -67, -48);
       loc[i].setFlag("acre.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Acre",
                                    "http://en.wikipedia.org/wiki/Acre_%28state%29"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Afeganisto", "Afghanistan"},
                             new String[] {"Kabul", "Kabul"},
                             +34, +31, +69, +12);
       loc[i].setFlag("afghanistan.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Afeganist%C3%A3o",
                                    "http://en.wikipedia.org/wiki/Afghanistan"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"frica do Sul", "South Africa"},
                             new String[] {"Pretria", "Pretoria"},
                             -25, -45, +28, +10);
       loc[i].setFlag("south_africa.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/%C3%81frica_do_Sul",
                                    "http://en.wikipedia.org/wiki/South_Africa"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Alagoas (Brasil)", "Alagoas (Brazil)"},
                             new String[] {"Macei", "Maceio"},
                             -9, -39, -35, -44);
       loc[i].setFlag("alagoas.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Alagoas",
                                    "http://en.wikipedia.org/wiki/Alagoas"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Albnia", "Albania"},
                             new String[] {"Tirana", "Tirana"},
                             +41, +20, +19, +50);
       loc[i].setFlag("albania.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Alb%C3%A2nia",
                                    "http://en.wikipedia.org/wiki/Albania"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Alemanha", "Germany"},
                             new String[] {"Berlim", "Berlin"},
                             +52, +30, +13, +25);
       loc[i].setFlag("germany.gif");
       loc[i].setContinent("europe");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Alemanha",
                                    "http://en.wikipedia.org/wiki/Germany"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Amap (Brasil)", "Amapa (Brazil)"},
                             new String[] {"Macap", "Macapa"},
                             +0, +2, -51, -3);
       loc[i].setFlag("amapa.gif");
       loc[i].setContinent("america");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Amap%C3%A1",
                                    "http://en.wikipedia.org/wiki/Amap%C3%A1"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Amazonas (Brasil)", "Amazonas (Brazil)"},
                             new String[] {"Manaus", "Manaus"},
                             -3, -6, -60, -1);
       loc[i].setFlag("amazonas.gif");
       loc[i].setContinent("america");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Amazonas",
                                    "http://en.wikipedia.org/wiki/Amazonas_State%2C_Brazil"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Andorra", "Andorra"},
                             new String[] {"Andorra a Velha", "Andorra La Vella"},
                             +42, +30, +1, +31);
       loc[i].setFlag("andorra.gif");
       loc[i].setContinent("europe");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Andorra",
                                    "http://en.wikipedia.org/wiki/Andorra"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Angola", "Angola"},
                             new String[] {"Luanda", "Luanda"},
                             -8, -48, +13, +14);
       loc[i].setFlag("angola.gif");
       loc[i].setContinent("africa");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Angola",
                                    "http://en.wikipedia.org/wiki/Angola"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Argentina", "Argentina"},
                             new String[] {"Buenos Aires", "Buenos Aires"},
                             -34, -35, -58, -30);
       loc[i].setFlag("argentina.gif");
       loc[i].setContinent("america");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Argentina",
                                    "http://en.wikipedia.org/wiki/Argentina"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Arglia", "Algeria"},
                             new String[] {"Argel", "Algiers"},
                             +35, +50, +3, +0);
       loc[i].setFlag("algeria.gif");
       loc[i].setContinent("africa");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Arg%C3%A9lia",
                                    "http://en.wikipedia.org/wiki/Algeria"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Arbia Saudita", "Saudi Arabia"},
                             new String[] {"Riade", "Riyadh"},
                             +24, +39, +46, +46);
       loc[i].setFlag("arabia.gif");
       loc[i].setContinent("asia");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Ar%C3%A1bia_Saudita",
                                    "http://en.wikipedia.org/wiki/Saudi_Arabia"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Armnia", "Armenia"},
                             new String[] {"Erevan", "Yerevan"},
                             +40, +16, +44, +34);
       loc[i].setFlag("armenia.gif");
       loc[i].setContinent("asia");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Arm%C3%A9nia",
                                    "http://en.wikipedia.org/wiki/Armenia"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"ustria", "Austria"},
                             new String[] {"Viena", "Vienna"},
                             +48, +12, +16, +22);
       loc[i].setFlag("austria.gif");
       loc[i].setContinent("europe");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/%C3%81ustria",
                                    "http://en.wikipedia.org/wiki/Austria"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Austrlia", "Australia"},
                             new String[] {"Canberra", "Camberra"},
                             -35, -15, +149, +28);
       loc[i].setFlag("australia.gif");
       loc[i].setContinent("oceania");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Austr%C3%A1lia",
                                    "http://en.wikipedia.org/wiki/Australia"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Azerbaijo", "Azerbaijan"},
                             new String[] {"Baku", "Baku"},
                             +40, +22, +49, +53);
       loc[i].setFlag("azerbaijan.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Azerbaij%C3%A3o",
                                    "http://en.wikipedia.org/wiki/Azerbaijan"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Bahia (Brasil)", "Bahia (Brazil)"},
                             new String[] {"Salvador", "Salvador"},
                             -12, -58, -38, -30);
       loc[i].setFlag("bahia.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Bahia",
                                    "http://en.wikipedia.org/wiki/Bahia"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Bahrain", "Bahrain"},
                             new String[] {"Manama", "Manama"},
                             +26, +13, +50, +35);
       loc[i].setFlag("bahrain.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Bahrein",
                                    "http://en.wikipedia.org/wiki/Bahrain"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Bangladesh", "Bangladesh"},
                             new String[] {"Dacca", "Dhaka"},
                             +23, +43, +90, +25);
       loc[i].setFlag("bangladesh.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Bangladesh",
                                    "http://en.wikipedia.org/wiki/Bangladesh"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Belize", "Belize"},
                             new String[] {"Belmopan", "Belmopan"},
                             +17, +15, -88, -46);
       loc[i].setFlag("belize.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Belize",
                                    "http://en.wikipedia.org/wiki/Belize"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Benin", "Benin"},
                             new String[] {"Porto Novo", "Porto-Novo"},
                             +6, +29, +2, +37);
       loc[i].setFlag("benin.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Benim",
                                    "http://en.wikipedia.org/wiki/Benin"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Bielorrssia", "Belarus"},
                             new String[] {"Minsk", "Minsk"},
                             +53, +55, +27, +33);
       loc[i].setFlag("belarus.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Bielorr%C3%BAssia",
                                    "http://en.wikipedia.org/wiki/Belarus"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Bolvia", "Bolivia"},
                             new String[] {"La Paz", "La Paz"},
                             -17, 0, -65, 0);
       loc[i].setFlag("bolivia.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Bol%C3%ADvia",
                                    "http://en.wikipedia.org/wiki/Bolivia"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Botswana", "Botswana"},
                             new String[] {"Gaborone", "Gaborone"},
                             -24, -45, +25, +55);
       loc[i].setFlag("botswana.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Botsuana",
                                    "http://en.wikipedia.org/wiki/Botswana"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Brasil", "Brazil"},
                             new String[] {"Braslia (DF)", "Braslia (DF)"},
                             -15, -8, -47, -9);
       loc[i].setFlag("brazil.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Brasil",
                                    "http://en.wikipedia.org/wiki/Brazil"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Brunei", "Brunei"},
                             new String[] {"Bandar Seri Begawan", "Bandar Seri Begawan"},
                             +4, +56, +114, +55);
       loc[i].setFlag("brunei.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Brunei",
                                    "http://en.wikipedia.org/wiki/Brunei"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Bulgria", "Bulgaria"},
                             new String[] {"Sofia", "Sofia"},
                             +42, +40, +23, +20);
       loc[i].setFlag("bulgaria.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Bulg%C3%A1ria",
                                    "http://en.wikipedia.org/wiki/Bulgaria"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Burkina Faso", "Burkina Faso"},
                             new String[] {"Ouagadougou", "Ouagadougou"},
                             +13, +0, -2, -0);
       loc[i].setFlag("burkina_faso.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Burquina_Faso",
                                    "http://en.wikipedia.org/wiki/Burkina_Faso"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Burundi", "Burundi"},
                             new String[] {"Bujumbura", "Bujumbura"},
                             -3, -30, +30, +0);
       loc[i].setFlag("burundi.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Burundi",
                                    "http://en.wikipedia.org/wiki/Burundi"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Buto", "Bhutan"},
                             new String[] {"Thimphu", "Thimphu"},
                             +27, +28, +89, +39);
       loc[i].setFlag("bhutan.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/But%C3%A3o",
                                    "http://en.wikipedia.org/wiki/Bhutan"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Blgica", "Belgium"},
                             new String[] {"Bruxelas", "Brussels"},
                             +50, +54, +4, +32);
       loc[i].setFlag("belgium.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/B%C3%A9lgica",
                                    "http://en.wikipedia.org/wiki/Belgium"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Bsnia", "Bosnia"},
                             new String[] {"Sarajevo", "Sarajevo"},
                             +43, +52, +18, +25);
       loc[i].setFlag("bosnia.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/B%C3%B3snia-Herzegovina",
                                    "http://en.wikipedia.org/wiki/Bosnia_and_Herzegovina"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Cabo Verde", "Cape Verde"},
                             new String[] {"Praia", "Praia"},
                             +14, +62, -23, -31);
       loc[i].setFlag("cape_verde.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Cabo_Verde",
                                    "http://en.wikipedia.org/wiki/Cape_Verde"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Camares", "Cameroon"},
                             new String[] {"Yaound", "Yaound"},
                             +3, +52, +11, +31);
       loc[i].setFlag("cameroon.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Camar%C3%B5es",
                                    "http://en.wikipedia.org/wiki/Cameroon"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Camboja", "Cambodia"},
                             new String[] {"Phnom Penh", "Phnom Penh"},
                             +11, +33, +104, +51);
       loc[i].setFlag("cambodia.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Camboja",
                                    "http://en.wikipedia.org/wiki/Cambodia"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Canada", "Canada"},
                             new String[] {"Ottawa", "Ottawa"},
                             +45, +24, -75, -40);
       loc[i].setFlag("canada.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Canad%C3%A1",
                                    "http://en.wikipedia.org/wiki/Canada"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Cazaquisto", "Kazakhstan"},
                             new String[] {"Astana", "Astana"},
                             +51, +10, +71, +30);
       loc[i].setFlag("kazakhstan.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Cazaquist%C3%A3o",
                                    "http://en.wikipedia.org/wiki/Kazakhstan"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Cear (Brasil)", "Ceara (Brazil)"},
                             new String[] {"Fortaleza", "Fortaleza"},
                             -3, -43, -38, -32);
       loc[i].setFlag("ceara.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Cear%C3%A1",
                                    "http://en.wikipedia.org/wiki/Ceara"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Chade", "Chad"},
                             new String[] {"Ndjamena", "Ndjamena"},
                             +12, +7, +15, +3);
       loc[i].setFlag("chad.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Chade",
                                    "http://en.wikipedia.org/wiki/Chad"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Chile", "Chile"},
                             new String[] {"Santiago", "Santiago"},
                             -33, -26, -70, -40);
       loc[i].setFlag("chile.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Chile",
                                    "http://en.wikipedia.org/wiki/Chile"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"China", "China"},
                             new String[] {"Pequim", "Beijing"},
                             +36, +55, +116, +23);
       loc[i].setFlag("china.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Rep%C3%BAblica_Popular_da_China",
                                    "http://en.wikipedia.org/wiki/People%27s_Republic_of_China"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Chipre", "Cyprus"},
                             new String[] {"Nicsia", "Nicosia"},
                             +35, +8, +33, +28);
       loc[i].setFlag("cyprus.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Chipre",
                                    "http://en.wikipedia.org/wiki/Cyprus"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Colmbia", "Colombia"},
                             new String[] {"Bogot", "Bogot"},
                             +4, +32, -74, -15);
       loc[i].setFlag("colombia.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Col%C3%B4mbia",
                                  "http://en.wikipedia.org/wiki/Colombia"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Coria do Norte", "North Korea"},
                             new String[] {"Pyongyang", "Pyongyang"},
                             +39, +2, +125, +45);
       loc[i].setFlag("north_korea.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Coreia_do_Norte",
                                    "http://en.wikipedia.org/wiki/North_Korea"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Coria do Sul", "South Korea"},
                             new String[] {"Seul", "Seoul"},
                             +37, +35, +127, +0);
       loc[i].setFlag("south_korea.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Coreia_do_Sul",
                                    "http://en.wikipedia.org/wiki/South_Korea"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Costa Rica", "Costa Rica"},
                             new String[] {"San Jos", "San Jos"},
                             +9, +56, -84, -5);
       loc[i].setFlag("costa_rica.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Costa_Rica",
                                    "http://en.wikipedia.org/wiki/Costa_Rica"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Costa do Marfim", "Ivory Coast"},
                             new String[] {"Yamoussoukro", "Yamoussokro"},
                             +6, +51, -5, -18);
       loc[i].setFlag("coast_ivory.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Costa_do_Marfim",
                                    "http://en.wikipedia.org/wiki/C%C3%B4te_d%27Ivoire"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Crocia", "Croatia"},
                             new String[] {"Zagreb", "Zagreb"},
                             +45, +48, +16, +0);
       loc[i].setFlag("croatia.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Cro%C3%A1cia",
                                    "http://en.wikipedia.org/wiki/Croatia"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Cuba", "Cuba"},
                             new String[] {"Havana", "Havana"},
                             +23, +0, -82, -23);
       loc[i].setFlag("cuba.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Cuba",
                                    "http://en.wikipedia.org/wiki/Cuba"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Dinamarca", "Denmark"},
                             new String[] {"Copenhague", "Copenhagen"},
                             +55, +40, +12, +34);
       loc[i].setFlag("denmark.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Dinamarca",
                                    "http://en.wikipedia.org/wiki/Denmark"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Djibouti", "Djibouti"},
                             new String[] {"Djibouti", "Djibouti"},
                             +11, +30, +43, +0);
       loc[i].setFlag("djibouti.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Jibuti",
                                    "http://en.wikipedia.org/wiki/Djibouti"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Egito", "Egypt"},
                             new String[] {"Cairo", "Cairo"},
                             +30, +2, +31, +21);
       loc[i].setFlag("egypt.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Egipto",
                                    "http://en.wikipedia.org/wiki/Egypt"});
       loc[i].setContinent("africa");

       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"El Salvador", "El Salvador"},
                             new String[] {"San Salvador", "San Salvador"},
                             +13, +42, -89, -13);
       loc[i].setFlag("el_salvador.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/El_Salvador",
                                    "http://en.wikipedia.org/wiki/El_Salvador"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Emirados rabes Unidos", "United Arab Emirates"},
                             new String[] {"Abu Dhabi", "Abu Dhabi"},
                             +24, +28, +54, +22);
       loc[i].setFlag("united_arab_emirates.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Emiratos_%C3%81rabes_Unidos",
                                    "http://en.wikipedia.org/wiki/United_Arab_Emirates"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Equador", "Ecuador"},
                             new String[] {"Quito", "Quito"},
                             -0, -13, -78, -32);
       loc[i].setFlag("ecuador.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Equador",
                                    "http://en.wikipedia.org/wiki/Ecuador"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Eritreia", "Eritrea"},
                             new String[] {"Asmara", "Asmara"},
                             +15, +0, +39, +0);
       loc[i].setFlag("eritrea.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Eritreia",
                                    "http://en.wikipedia.org/wiki/Eritrea"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Esccia", "Scotland"},
                             new String[] {"Edimburgo", "Edinburgh"},
                             +55, +55, -3, -10);
       loc[i].setFlag("scotland.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Esc%C3%B3cia",
                                    "http://en.wikipedia.org/wiki/Scotland"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Eslovquia", "Slovakia"},
                             new String[] {"Bratislava", "Bratislava"},
                             +48, +9, +17, +7);
       loc[i].setFlag("slovakia.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Eslov%C3%A1quia",
                                    "http://en.wikipedia.org/wiki/Slovakia"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Eslovnia", "Slovenia"},
                             new String[] {"Ljubljana", "Ljubljana"},
                             +46, +3, +14, +31);
       loc[i].setFlag("slovenia.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Eslov%C3%A9nia",
                                    "http://en.wikipedia.org/wiki/Slovenia"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Espanha", "Spain"},
                             new String[] {"Madrid", "Madrid"},
                             +40, +26, -3, -42);
       loc[i].setFlag("spain.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Espanha",
                                    "http://en.wikipedia.org/wiki/Spain"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Esprito Santo (Brasil)", "Espirito Santo (Brazil)"},
                             new String[] {"Vitria", "Vitoria"},
                             -20, -19, -40, -20);
       loc[i].setFlag("espirito_santo.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Estado_do_Espirito_Santo",
                                    "http://en.wikipedia.org/wiki/Espirito_Santo"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Estados Unidos", "United States"},
                             new String[] {"Washington", "Washington"},
                             +38, +53, -77, -2);
       loc[i].setFlag("united_states.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Estados_Unidos_da_Am%C3%A9rica",
                                    "http://en.wikipedia.org/wiki/United_States"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Estnia", "Estonia"},
                             new String[] {"Tallinn", "Tallinn"},
                             +59, +25, +24, +45);
       loc[i].setFlag("estonia.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Est%C3%B3nia",
                                    "http://en.wikipedia.org/wiki/Estonia"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Etipia", "Ethiopia"},
                             new String[] {"Addis Abeba", "Addis Abeba"},
                             +9, +2, +38, +42);
       loc[i].setFlag("ethiopia.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Eti%C3%B3pia",
                                    "http://en.wikipedia.org/wiki/Ethiopia"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Filipinas", "Philippines"},
                             new String[] {"Manila", "Manila"},
                             +14, +35, +121, +0);
       loc[i].setFlag("philippines.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Filipinas",
                                    "http://en.wikipedia.org/wiki/Philippines"});
       loc[i].setContinent("oceania");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Finlndia", "Finland"},
                             new String[] {"Helsnquia", "Helsinki"},
                             +60, +10, +24, +58);
       loc[i].setFlag("finland.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Finl%C3%A2ndia",
                                    "http://en.wikipedia.org/wiki/Finland"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Frana", "France"},
                             new String[] {"Paris", "Paris"},
                             +48, +48, +2, +20);
       loc[i].setFlag("france.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Fran%C3%A7a",
                                    "http://en.wikipedia.org/wiki/France"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Gabo", "Gabon"},
                             new String[] {"Libreville", "Libreville"},
                             +0, +30, +9, +32);
       loc[i].setFlag("gabon.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Gab%C3%A3o",
                                    "http://en.wikipedia.org/wiki/Gabon"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Gmbia", "Gambia"},
                             new String[] {"Banjul", "Banjul"},
                             +13, +28, -16, -39);
       loc[i].setFlag("gambia.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/G%C3%A2mbia",
                                    "http://en.wikipedia.org/wiki/The_Gambia"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;


       loc[i] = new Location(new String[] {"Gana", "Ghana"},
                             new String[] {"Acra", "Accra"},
                             +5, +33, -0, -13);
       loc[i].setFlag("ghana.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Gana",
                                    "http://en.wikipedia.org/wiki/Ghana"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Gergia", "Georgia"},
                             new String[] {"Tibilisi", "T`bilisi"},
                             +41, +43, +44, +49);
       loc[i].setFlag("georgia.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Ge%C3%B3rgia",
                                    "http://en.wikipedia.org/wiki/Georgia_%28country%29"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Gois (Brasil)", "Goias (Brazil)"},
                             new String[] {"Goinia", "Goiania"},
                             -16, -40, -49, -15);
       loc[i].setFlag("goias.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Goi%C3%A1s",
                                    "http://en.wikipedia.org/wiki/Goias"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Grcia", "Greece"},
                             new String[] {"Atenas", "Athens"},
                             +37, +58, +23, +43);
       loc[i].setFlag("greece.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Gr%C3%A9cia",
                                    "http://en.wikipedia.org/wiki/Greece"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Guatemala", "Guatemala"},
                             new String[] {"Cidade da Guatemala", "Guatemala City"},
                             +14, +37, -90, -31);
       loc[i].setFlag("guatemala.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Guatemala",
                                    "http://en.wikipedia.org/wiki/Guatemala"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Guiana", "Guyana"},
                             new String[] {"Georgetown", "Georgetown"},
                             +6, +45, -58, -15);
       loc[i].setFlag("guyana.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Guiana",
                                    "http://en.wikipedia.org/wiki/Guyana"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Guin", "Guinea"},
                             new String[] {"Conakry", "Conakry"},
                             +9, +31, -13, -43);
       loc[i].setFlag("guinea.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Guin%C3%A9",
                                    "http://en.wikipedia.org/wiki/Guinea"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Guin-Bissau", "Guinea-Bissau"},
                             new String[] {"Bissau", "Bissau"},
                             +11, +51, -15, -35);
       loc[i].setFlag("guinea-bissau.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Guin%C3%A9-Bissau",
                                    "http://en.wikipedia.org/wiki/Guinea-Bissau"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Guin-Equatorial", "Equatorial-Guinea"},
                             new String[] {"Malabo", "Malabo"},
                             +1, +25, +9, +57);
       loc[i].setFlag("equatorial_guinea.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Guin%C3%A9_Equatorial",
                                    "http://en.wikipedia.org/wiki/Equatorial_Guinea"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Haiti", "Haiti"},
                             new String[] {"Porto Prncipe", "Port-au-Prince"},
                             +18, +33, -72, -20);
       loc[i].setFlag("haiti.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Haiti",
                                    "http://en.wikipedia.org/wiki/Haiti"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Hava", "Hawaii"},
                             new String[] {"Honolulu", "Honolulu"},
                             +21, +18, -157, -51);
       loc[i].setFlag("hawaii.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Hava%C3%AD",
                                    "http://en.wikipedia.org/wiki/Hawaii"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Holanda", "Netherlands"},
                             new String[] {"Amsterd", "Amsterdam"},
                             +52, +22, +4, +53);
       loc[i].setFlag("netherlands.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Pa%C3%ADses_Baixos",
                                    "http://en.wikipedia.org/wiki/Netherlands"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Honduras", "Honduras"},
                             new String[] {"Tegucigalpa", "Tegucigalpa"},
                             +14, +6, -87, -13);
       loc[i].setFlag("honduras.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Honduras",
                                    "http://en.wikipedia.org/wiki/Honduras"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Hungria", "Hungary"},
                             new String[] {"Budapeste", "Budapest"},
                             +47, +30, +19, +15);
       loc[i].setFlag("hungary.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Hungria",
                                    "http://en.wikipedia.org/wiki/Hungary"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Imen", "Yemen"},
                             new String[] {"Sana", "Sanaa"},
                             +15, +24, +44, +14);
       loc[i].setFlag("yemen.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/I%C3%A9men",
                                    "http://en.wikipedia.org/wiki/Yemen"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"ndia", "India"},
                             new String[] {"Nova Deli", "New Delhi"},
                             +28, +35, +77, +12);
       loc[i].setFlag("india.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/%C3%8Dndia",
                                    "http://en.wikipedia.org/wiki/India"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Indonsia", "Indonesia"},
                             new String[] {"Jacarta", "Jakarta"},
                             -6, -8, +106, +45);
       loc[i].setFlag("indonesia.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Indon%C3%A9sia",
                                    "http://en.wikipedia.org/wiki/Indonesia"});
       loc[i].setContinent("oceania");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Inglaterra", "England"},
                             new String[] {"Londres", "London"},
                             +51, +32, -0, -5);
       loc[i].setFlag("england.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Reino_Unido",
                                    "http://en.wikipedia.org/wiki/United_Kingdom"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Iraque", "Iraq"},
                             new String[] {"Baghdad", "Baghdad"},
                             +33, +20, +44, +26);
       loc[i].setFlag("iraq.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Iraque",
                                    "http://en.wikipedia.org/wiki/Iraq"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Irlanda", "Ireland"},
                             new String[] {"Dublin", "Dublin"},
                             +53, +26, -6, -15);
       loc[i].setFlag("ireland.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Rep%C3%BAblica_da_Irlanda",
                                    "http://en.wikipedia.org/wiki/Republic_of_Ireland"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Ir", "Iran"},
                             new String[] {"Teer", "Teheran"},
                             +35, +45, +51, +25);
       loc[i].setFlag("iran.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Ir%C3%A3o",
                                    "http://en.wikipedia.org/wiki/Iran"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Islndia", "Iceland"},
                             new String[] {"Reykjavik", "Reykjavik"},
                             +64, +9, -21, -51);
       loc[i].setFlag("iceland.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Isl%C3%A2ndia",
                                    "http://en.wikipedia.org/wiki/Iceland"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Israel", "Israel"},
                             new String[] {"Jerusalm", "Jerusalem"},
                             +31, +47, +35, +13);
       loc[i].setFlag("israel.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Israel",
                                    "http://en.wikipedia.org/wiki/Israel"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Itlia", "Italy"},
                             new String[] {"Roma", "Rome"},
                             +41, +48, +12, +36);
       loc[i].setFlag("italy.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/It%C3%A1lia",
                                    "http://en.wikipedia.org/wiki/Italy"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Jamaica", "Jamaica"},
                             new String[] {"Kingston", "Kingston"},
                             +17, +59, -76, -49);
       loc[i].setFlag("jamaica.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Jamaica",
                                    "http://en.wikipedia.org/wiki/Jamaica"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Japo", "Japan"},
                             new String[] {"Tquio", "Tokyo"},
                             +35, +40, +139, +45);
       loc[i].setFlag("japan.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Jap%C3%A3o",
                                    "http://en.wikipedia.org/wiki/Japan"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Jordnia", "Jordan"},
                             new String[] {"Amman", "Amman"},
                             +31, +57, +35, +56);
       loc[i].setFlag("jordan.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Jord%C3%A2nia",
                                    "http://en.wikipedia.org/wiki/Jordan"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Knia", "Kenya"},
                             new String[] {"Nairobi", "Nairobi"},
                             -1, -25, +36, +55);
       loc[i].setFlag("kenya.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Qu%C3%A9nia",
                                    "http://en.wikipedia.org/wiki/Kenya"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Kuwait", "Kuwait"},
                             new String[] {"Cidade do Kuwait", "Kuwait City"},
                             +29, +20, +48, +0);
       loc[i].setFlag("kuwait.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Couaite",
                                    "http://en.wikipedia.org/wiki/Kuwait"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Laos", "Laos"},
                             new String[] {"Vienciana", "Vientiane"},
                             +18, +58, +102, +36);
       loc[i].setFlag("laos.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Laus",
                                    "http://en.wikipedia.org/wiki/Laos"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Lesoto", "Lesotho"},
                             new String[] {"Maseru", "Maseru"},
                             -29, -18, +27, +28);
       loc[i].setFlag("lesotho.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Lesoto",
                                    "http://en.wikipedia.org/wiki/Lesotho"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Letnia", "Latvia"},
                             new String[] {"Riga", "Riga"},
                             +56, +58, +24, +8);
       loc[i].setFlag("latvia.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Let%C3%B3nia",
                                    "http://en.wikipedia.org/wiki/Latvia"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Lbano", "Lebanon"},
                             new String[] {"Beirute", "Beirut"},
                             +33, +54, +35, +32);
       loc[i].setFlag("lebanon.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/L%C3%ADbano",
                                    "http://en.wikipedia.org/wiki/Lebanon"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Libria", "Liberia"},
                             new String[] {"Monrvia", "Monrovia"},
                             +6, +37, -10, -47);
       loc[i].setFlag("liberia.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Lib%C3%A9ria",
                                    "http://en.wikipedia.org/wiki/Liberia"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Lbia", "Libya"},
                             new String[] {"Trpoli", "Tripoli"},
                             +31, +34, +12, +44);
       loc[i].setFlag("libya.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/L%C3%ADbia",
                                    "http://en.wikipedia.org/wiki/Libya"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Liechtenstein", "Liechtenstein"},
                             new String[] {"Vaduz", "Vaduz"},
                             +47, +0, +10, +0);
       loc[i].setFlag("liechtenstein.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Listenstaina",
                                    "http://en.wikipedia.org/wiki/Liechtenstein"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Lithunia", "Lithuania"},
                             new String[] {"Vilnius", "Vilnius"},
                             +54, +41, +25, +19);
       loc[i].setFlag("lithuania.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Litu%C3%A2nia",
                                    "http://en.wikipedia.org/wiki/Lithuania"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Luxemburgo", "Luxembourg"},
                             new String[] {"Luxemburgo", "Luxembourg"},
                             +49, +36, +6, +7);
       loc[i].setFlag("luxembourg.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Luxemburgo",
                                    "http://en.wikipedia.org/wiki/Luxembourg"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Macednia", "Macedonia"},
                             new String[] {"Skopje", "Skopje"},
                             +42, +0, +21, +26);
       loc[i].setFlag("macedonia.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Maced%C3%B3nia",
                                    "http://en.wikipedia.org/wiki/Republic_of_Macedonia"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Madagascar", "Madagascar"},
                             new String[] {"Antananarivo", "Antananarivo"},
                             -18, -55, +47, +31);
       loc[i].setFlag("madagascar.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Madag%C3%A1scar",
                                    "http://en.wikipedia.org/wiki/Madagascar"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Malsia", "Malaysia"},
                             new String[] {"Kuala Lumpur", "Kuala Lumpur"},
                            +3, +8, +101, +42);
       loc[i].setFlag("malaysia.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Mal%C3%A1sia",
                                    "http://en.wikipedia.org/wiki/Malaysia"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Malawi", "Malawi"},
                            new String[] {"Lilongwe", "Lilongwe"},
                            -13, -57, +33, +42);
       loc[i].setFlag("malawi.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Malaui",
                                    "http://en.wikipedia.org/wiki/Malawi"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Mali", "Mali"},
                             new String[] {"Bamaco", "Bamako"},
                             +12, +39, -8, -0);
       loc[i].setFlag("mali.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Mali",
                                    "http://en.wikipedia.org/wiki/Mali"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Maranho (Brasil)", "Maranhao (Brazil)"},
                             new String[] {"So Lus", "So Luis"},
                             -2, -31, -44, -18);
       loc[i].setFlag("maranhao.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Maranh%C3%A3o",
                                    "http://en.wikipedia.org/wiki/Maranhao"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Marrocos", "Morocco"},
                             new String[] {"Rabat", "Rabat"},
                             +34, +0, -6, -51);
       loc[i].setFlag("morocco.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Marrocos",
                                    "http://en.wikipedia.org/wiki/Morocco"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Mato Grosso (Brasil)", "Mato Grosso (Brazil)"},
                             new String[] {"Cuiab", "Cuiaba"},
                             -15, -35, -56, -5);
       loc[i].setFlag("mato_grosso.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Mato_Grosso",
                                    "http://en.wikipedia.org/wiki/Mato_Grosso"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Mato Grosso do Sul (Brasil)", "Mato Grosso do Sul (Brazil)"},
                             new String[] {"Campo Grande", "Campo Grande"},
                             -20, -26, -54, -38);
       loc[i].setFlag("mato_grosso_do_sul.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Mato_Grosso_do_Sul",
                                    "http://en.wikipedia.org/wiki/Mato_Grosso_do_Sul"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Mauritnia", "Mauritania"},
                             new String[] {"Nouakchott", "Nouakchott"},
                             +18, +9, -15, -60);
       loc[i].setFlag("mauritania.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Maurit%C3%A2nia",
                                    "http://en.wikipedia.org/wiki/Mauritania"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Mxico", "Mexico"},
                             new String[] {"Cidade do Mxico", "Mexico City"},
                             +19, +0, -99, -22);
       loc[i].setFlag("mexico.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/M%C3%A9xico",
                                    "http://en.wikipedia.org/wiki/Mexico"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Minas Gerais (Brasil)", "Minas Gerais (Brazil)"},
                             new String[] {"Belo Horizonte", "Belo Horizonte"},
                             -19, -55, -43, -56);
       loc[i].setFlag("minas_gerais.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Minas_Gerais",
                                    "http://en.wikipedia.org/wiki/Minas_Gerais"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Moambique", "Mozambique"},
                             new String[] {"Maputo", "Maputo"},
                             -25, -57, +32, +35);
       loc[i].setFlag("mozambique.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Mo%C3%A7ambique",
                                    "http://en.wikipedia.org/wiki/Mozambique"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Moldvia", "Moldova"},
                             new String[] {"Chisinu", "Chisinu"},
                             +47, +0, +28, +55);
       loc[i].setFlag("moldova.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Mold%C3%A1via",
                                    "http://en.wikipedia.org/wiki/Republic_of_Moldova"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Monglia", "Mongolia"},
                             new String[] {"Ulaanbaatar", "Ulaanbaatar"},
                             +47, +55, +106, +53);
       loc[i].setFlag("mongolia.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Mong%C3%B3lia",
                                    "http://en.wikipedia.org/wiki/Mongolia"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Myanmar", "Myanmar"},
                             new String[] {"Pyinmana", "Pyinmana"},
                             +19, +44, +96, +13);
       loc[i].setFlag("myanmar.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Mianmar",
                                    "http://en.wikipedia.org/wiki/Myanmar"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Nambia", "Namibia"},
                             new String[] {"Windhoek", "Windhoek"},
                             -22, -34, +17, +6);
       loc[i].setFlag("namibia.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Nam%C3%ADbia",
                                    "http://en.wikipedia.org/wiki/Namibia"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Nepal", "Nepal"},
                             new String[] {"kathmandu", "Kathmandu"},
                             +27, +42, +85, +19);
       loc[i].setFlag("nepal.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Nepal",
                                    "http://en.wikipedia.org/wiki/Nepal"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Nicargua", "Nicaragua"},
                             new String[] {"Mangua", "Managua"},
                             +12, +6, -86, -18);
       loc[i].setFlag("nicaragua.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Nicar%C3%A1gua",
                                    "http://en.wikipedia.org/wiki/Nicaragua"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Nger", "Niger"},
                             new String[] {"Niamey", "Niamey"},
                             +13, +31, +2, +7);
       loc[i].setFlag("niger.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/N%C3%ADger",
                                    "http://en.wikipedia.org/wiki/Niger"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Nigria", "Nigeria"},
                             new String[] {"Abuja", "Abuja"},
                             +9, +12, +7, +11);
       loc[i].setFlag("nigeria.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Nig%C3%A9ria",
                                    "http://en.wikipedia.org/wiki/Nigeria"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Noruega", "Norway"},
                             new String[] {"Oslo", "Oslo"},
                             +59, +56, +10, +42);
       loc[i].setFlag("norway.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Noruega",
                                    "http://en.wikipedia.org/wiki/Norway"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Nova Calednia", "New Caledonia"},
                             new String[] {"Noumea", "Noumea"},
                             -22, -8, +166, +39);
       loc[i].setFlag("new_caledonia.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Nova_Caled%C3%B4nia",
                                    "http://en.wikipedia.org/wiki/New_Caledonia"});
       loc[i].setContinent("oceania");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Nova Zelndia", "New Zealand"},
                             new String[] {"Wellington", "Wellington"},
                             -41, -17, +174, +46);
       loc[i].setFlag("new_zealand.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Nova_Zel%C3%A2ndia",
                                    "http://en.wikipedia.org/wiki/New_Zealand"});
       loc[i].setContinent("oceania");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Om", "Oman"},
                             new String[] {"Muscat", "Muscat"},
                             +23, +14, +57, +58);
       loc[i].setFlag("oman.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Om%C3%A3",
                                    "http://en.wikipedia.org/wiki/Oman"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Palestina", "Palestine"},
                             new String[] {"Jerusalm", "Jerusalem"},
                             +31, +47, +35, +13);
       loc[i].setFlag("palestine.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Palestina",
                                    "http://en.wikipedia.org/wiki/Palestinian_territories"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Panam", "Panama"},
                             new String[] {"Cidade do Panam", "Panama City"},
                             +8, +58, -79, -32);
       loc[i].setFlag("panama.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Panam%C3%A1",
                                    "http://en.wikipedia.org/wiki/Panama"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Papua - Nova Guin", "Papua New Guinea "},
                             new String[] {"Port Moresby", "Port Moresby"},
                             -9, -0, +147, +0);
       loc[i].setFlag("papua_new_guinea.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Papua_Nova_Guin%C3%A9",
                                    "http://en.wikipedia.org/wiki/Papua_New_Guinea"});
       loc[i].setContinent("oceania");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Paquisto", "Pakistan"},
                             new String[] {"Islamabad", "Islamabad"},
                             +33, +40, +73, +8);
       loc[i].setFlag("pakistan.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Paquist%C3%A3o",
                                    "http://en.wikipedia.org/wiki/Pakistan"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Par (Brasil)", "Par (Brazil)"},
                             new String[] {"Belm", "Belem"},
                             -1, -27, -48, -30);
       loc[i].setFlag("para.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Par%C3%A1",
                                    "http://en.wikipedia.org/wiki/Par%C3%A1"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Paraguai", "Paraguay"},
                             new String[] {"Assuno", "Asuncin"},
                             -25, -16, -57, -40);
       loc[i].setFlag("paraguay.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Paraguai",
                                    "http://en.wikipedia.org/wiki/Paraguay"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Paran (Brasil)", "Parana (Brazil)"},
                             new String[] {"Curitiba", "Curitiba"},
                             -25, -25, -49, -16);
       loc[i].setFlag("parana.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Paran%C3%A1",
                                    "http://en.wikipedia.org/wiki/Parana"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Paraba (Brasil)", "Paraiba (Brazil)"},
                             new String[] {"Joo Pessoa", "Joo Pessoa"},
                             -7, -6, -34, -51);
       loc[i].setFlag("paraiba.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Para%C3%ADba",
                                    "http://en.wikipedia.org/wiki/Paraiba"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Pernambuco (Brasil)", "Pernambuco (Brazil)"},
                             new String[] {"Recife", "Recife"},
                             -8, -3, -34, -52);
       loc[i].setFlag("pernambuco.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Pernambuco",
                                    "http://en.wikipedia.org/wiki/Pernambuco"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Peru", "Peru"},
                             new String[] {"Lima", "Lime"},
                             -11, -57, -76, -43);
       loc[i].setFlag("peru.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Peru",
                                    "http://en.wikipedia.org/wiki/Peru"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Piau (Brasil)", "Piaui (Brazil)"},
                             new String[] {"Teresina", "Teresina"},
                             -5, -5, -42, -48);
       loc[i].setFlag("piaui.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Piau%C3%AD",
                                    "http://en.wikipedia.org/wiki/Piaui"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Polnia", "Poland"},
                             new String[] {"Varsvia", "Warsaw"},
                             +52, +14, +21, +0);
       loc[i].setFlag("poland.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Pol%C3%B3nia",
                                    "http://en.wikipedia.org/wiki/Poland"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Porto Rico", "Puerto Rico"},
                             new String[] {"San Juan", "San Juan"},
                             +18, +28, -66, -7);
       loc[i].setFlag("porto_rico.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Porto_Rico",
                                    "http://en.wikipedia.org/wiki/Puerto_Rico"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Portugal", "Portugal"},
                             new String[] {"Lisboa", "Lisbon"},
                             +38, +38, -9, -6);
       loc[i].setFlag("portugal.gif");
       loc[i].setContinent("europe");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Portugal",
                                    "http://en.wikipedia.org/wiki/Portugal"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Qatar", "Qatar"},
                             new String[] {"Doha", "Doha"},
                             +25, +17, +51, +32);
       loc[i].setFlag("qatar.gif");
       loc[i].setContinent("asia");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Catar",
                                    "http://en.wikipedia.org/wiki/Qatar"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Quirguisto", "Kyrgyzstan"},
                             new String[] {"Bishkek", "Bishkek"},
                             +42, +54, +74, +36);
       loc[i].setFlag("kyrgyzstan.gif");
       loc[i].setContinent("asia");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Quirguist%C3%A3o",
                                    "http://en.wikipedia.org/wiki/Kyrgyzstan"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Repblica Centro Africana", "Central African Republic"},
                             new String[] {"Bangui", "Bangui"},
                             +4, +22, +18, +35);
       loc[i].setFlag("central_african_republic.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Rep%C3%BAblica_Centro-Africana",
                                    "http://en.wikipedia.org/wiki/Central_African_Republic"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Repblica Democrtica do Congo", "Democratic Republic of the Congo"},
                             new String[] {"Kinshasa", "Kinshasa"},
                             -4, -24, +15, +24);
       loc[i].setFlag("congo_democratic.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Rep%C3%BAblica_Democr%C3%A1tica_do_Congo",
                                    "http://en.wikipedia.org/wiki/Democratic_Republic_of_the_Congo"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Repblica do Congo", "Republic of the Congo"},
                             new String[] {"Brazzaville", "Brazzaville"},
                             -4, -10, +15, +14);
       loc[i].setFlag("republic_congo.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Rep%C3%BAblica_do_Congo",
                                    "http://en.wikipedia.org/wiki/Republic_of_the_Congo"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Repblica Dominicana", "Dominican Republic"},
                             new String[] {"Santo Domingo", "Santo Domingo"},
                             +18, +29, -69, -54);
       loc[i].setFlag("dominican_republic.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Rep%C3%BAblica_Dominicana",
                                    "http://en.wikipedia.org/wiki/Dominican_Republic"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Repblica Tcheca", "Czech Republic"},
                             new String[] {"Praga", "Prague"},
                             +50, +5, +14, +28);
       loc[i].setFlag("czech_republic.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Rep%C3%BAblica_Checa",
                                    "http://en.wikipedia.org/wiki/Czech_Republic"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Rio Grande do Norte (Brasil)", "Rio Grande do Norte (Brazil)"},
                             new String[] {"Natal", "Natal"},
                             -5, -47, -35, -12);
       loc[i].setFlag("rio_grande_do_norte.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Rio_Grande_do_Norte",
                                    "http://en.wikipedia.org/wiki/Rio_Grande_do_Norte"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Rio Grande do Sul (Brasil)", "Rio Grande do Sul (Brazil)"},
                             new String[] {"Porto Alegre", "Porto Alegre"},
                             -30, -1, -51, -13);
       loc[i].setFlag("rio_grande_do_sul.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Rio_Grande_do_Sul",
                                    "http://en.wikipedia.org/wiki/Rio_Grande_do_Sul"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Rio de Janeiro (Brasil)", "Rio de Janeiro (Brazil)"},
                             new String[] {"Rio de Janeiro", "Rio de Janeiro"},
                             -22, -57, -43, -12);
       loc[i].setFlag("rio_de_janeiro.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Rio_de_Janeiro",
                                    "http://en.wikipedia.org/wiki/Rio_de_Janeiro"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Romnia", "Romania"},
                             new String[] {"Bucareste", "Bucharest"},
                             +44, +26, +26, +6);
       loc[i].setFlag("romania.gif");
       loc[i].setContinent("europe");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Rom%C3%A9nia",
                                    "http://en.wikipedia.org/wiki/Romania"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Rondnia (Brasil)", "Rondnia (Brazil)"},
                             new String[] {"Porto Velho", "Porto Velho"},
                             -8, -45, -63, -54);
       loc[i].setFlag("rondonia.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Rond%C3%B4nia",
                                    "http://en.wikipedia.org/wiki/Rondonia"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Roraima (Brasil)", "Roraima (Brazil)"},
                             new String[] {"Boa Vista", "Boa Vista"},
                             +2, +49, -60, -40);
       loc[i].setFlag("roraima.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Roraima",
                                    "http://en.wikipedia.org/wiki/Roraima"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Ruanda", "Rwanda"},
                             new String[] {"Kigali", "Kigali"},
                             -1, -57, +30, +4);
       loc[i].setFlag("rwanda.gif");
       loc[i].setContinent("africa");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Ruanda",
                                    "http://en.wikipedia.org/wiki/Rwanda"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Rssia", "Russia"},
                             new String[] {"Moscou", "Moscow"},
                             +55, +45, +37, +34);
       loc[i].setFlag("russia.gif");
       loc[i].setContinent("asia");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/R%C3%BAssia",
                                    "http://en.wikipedia.org/wiki/Russia"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Saara Ocidental", "Western Sahara"},
                             new String[] {"El Aain", "El Aain"},
                             +27, +0, -13, -12);
       loc[i].setFlag("western_sahara.gif");
       loc[i].setContinent("africa");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Sara_Ocidental",
                                    "http://en.wikipedia.org/wiki/Western_Sahara"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Santa Catarina (Brasil)", "Santa Catarina (Brazil)"},
                             new String[] {"Florianpolis", "Florianopolis"},
                             -27, -35, -48, -32);
       loc[i].setFlag("santa_catarina.gif");
       loc[i].setContinent("america");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Santa_Catarina",
                                    "http://en.wikipedia.org/wiki/Santa_Catarina_%28state%29"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"So Paulo (Brasil)", "Sao Paulo (Brazil)"},
                             new String[] {"So Paulo", "Sao Paulo"},
                             -23, -32, -46, -38);
       loc[i].setFlag("sao_paulo.gif");
       loc[i].setContinent("america");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/S%C3%A3o_Paulo",
                                    "http://en.wikipedia.org/wiki/S%C3%A3o_Paulo"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Senegal", "Senegal"},
                             new String[] {"Dakar", "Dakar"},
                             +14, +44, -17, -21);
       loc[i].setFlag("senegal.gif");
       loc[i].setContinent("africa");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Senegal",
                                    "http://en.wikipedia.org/wiki/Senegal"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Sergipe (Brasil)", "Sergipe (Brazil)"},
                             new String[] {"Aracaju", "Aracaju"},
                             -10, -54, -37, -4);
       loc[i].setFlag("sergipe.gif");
       loc[i].setContinent("america");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Sergipe",
                                    "http://en.wikipedia.org/wiki/Sergipe"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Serra Leoa", "Sierra Leone"},
                             new String[] {"Freetown", "Freetown"},
                             +8, +30, -13, -1);
       loc[i].setFlag("sierra_leone.gif");
       loc[i].setContinent("africa");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Serra_Leoa",
                                    "http://en.wikipedia.org/wiki/Sierra_Leone"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Srvia e Montenegro", "Serbia and Montenegro"},
                             new String[] {"Belgrado", "Belgrade"},
                             +44, +52, +20, +32);
       loc[i].setFlag("serbia_and_montenegro.gif");
       loc[i].setContinent("europe");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/S%C3%A9rvia_e_Montenegro",
                                    "http://en.wikipedia.org/wiki/Serbia_and_Montenegro"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Singapura", "Singapore"},
                             new String[] {"Cidade de Singapura", "Singapore"},
                              +1, +40, +103, +51);
       loc[i].setFlag("singapore.gif");
       loc[i].setContinent("asia");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Singapura",
                                    "http://en.wikipedia.org/wiki/Singapore"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Sria", "Syria"},
                             new String[] {"Damasco", "Damascus"},
                             +33, +30, +36, +18);
       loc[i].setFlag("syria.gif");
       loc[i].setContinent("asia");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/S%C3%ADria",
                                    "http://en.wikipedia.org/wiki/Syria"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Somlia", "Somalia"},
                             new String[] {"Mogadscio", "Mogadishu"},
                             +2, +2, +45, +21);
       loc[i].setFlag("somalia.gif");
       loc[i].setContinent("africa");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Som%C3%A1lia",
                                    "http://en.wikipedia.org/wiki/Somalia"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Sri Lanka", "Sri Lanka"},
                             new String[] {"Colombo", "Colombo"},
                             +6, +17, +80, +19);
       loc[i].setFlag("sri_lanka.gif");
       loc[i].setContinent("asia");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Seri_Lanca",
                                    "http://en.wikipedia.org/wiki/Sri_Lanka"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Suazilndia", "Swaziland"},
                             new String[] {"Mbabane", "Mbabane"},
                             -26, -19, +31, +8);
       loc[i].setFlag("swaziland.gif");
       loc[i].setContinent("africa");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Suazil%C3%A2ndia",
                                    "http://en.wikipedia.org/wiki/Swaziland"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Sudo", "Sudan"},
                             new String[] {"Khartoum", "Khartoum"},
                             +15, +0, +32, +32);
       loc[i].setFlag("sudan.gif");
       loc[i].setContinent("africa");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Sud%C3%A3o",
                                    "http://en.wikipedia.org/wiki/Sudan"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Sucia", "Sweden"},
                             new String[] {"Estocolmo", "Stockholm"},
                             +59, +19, +18, +3);
       loc[i].setFlag("sweden.gif");
       loc[i].setContinent("europe");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Su%C3%A9cia",
                                    "http://en.wikipedia.org/wiki/Sweden"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Sua", "Switzerland"},
                             new String[] {"Berna", "Bern"},
                             +46, +57, +7, +26);
       loc[i].setFlag("switzerland.gif");
       loc[i].setContinent("europe");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Su%C3%AD%C3%A7a",
                                    "http://en.wikipedia.org/wiki/Switzerland"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Suriname", "Suriname"},
                             new String[] {"Paramaribo", "Paramaribo"},
                             +5, +45, -55, -15);
       loc[i].setFlag("suriname.gif");
       loc[i].setContinent("america");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Suriname",
                                    "http://en.wikipedia.org/wiki/Suriname"});
       locChoice.add(loc[i].getCountryName(language));


       loc[i] = new Location(new String[] {"Tailndia", "Thailand"},
                             new String[] {"Bancoc", "Bangkok"},
                             +13, +45, +100, +30);
       loc[i].setFlag("thailand.gif");
       loc[i].setContinent("asia");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Tail%C3%A2ndia",
                                    "http://en.wikipedia.org/wiki/Thailand"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Tadjiquisto", "Tajikistan"},
                             new String[] {"Dushanbe", "Dushanbe"},
                             +38, +35, +68, +48);
       loc[i].setFlag("tajikistan.gif");
       loc[i].setContinent("asia");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Tajiquist%C3%A3o",
                                    "http://en.wikipedia.org/wiki/Tajikistan"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Tanznia", "Tanzania"},
                             new String[] {"Dodoma", "Dodoma"},
                             -6, -15, +35, +45);
       loc[i].setFlag("tanzania.gif");
       loc[i].setContinent("africa");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Tanz%C3%A2nia",
                                    "http://en.wikipedia.org/wiki/Tanzania"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Timor Leste", "East Timor"},
                             new String[] {"Dili", "Dili"},
                             -8, -57, +125, +58);
       loc[i].setFlag("east_timor.gif");
       loc[i].setContinent("oceania");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Timor_Leste",
                                    "http://en.wikipedia.org/wiki/East_Timor"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Tocantins (Brasil)", "Tocantins (Brazil)"},
                             new String[] {"Palmas", "Palmas"},
                             -10, -12, -48, -21);
       loc[i].setFlag("tocantins.gif");
       loc[i].setContinent("america");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Tocantins",
                                    "http://en.wikipedia.org/wiki/Tocantins_%28state%29"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Togo", "Togo"},
                             new String[] {"Lom", "Lome"},
                             +6, +8, +1, +13);
       loc[i].setFlag("togo.gif");
       loc[i].setContinent("africa");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Togo",
                                    "http://en.wikipedia.org/wiki/Togo"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Trinidad e Tobago", "Trinidad and Tobago"},
                             new String[] {"Port of Spain", "Port of Spain"},
                             +10, +40, -61, -31);
       loc[i].setFlag("trinidad_and_tobago.gif");
       loc[i].setContinent("america");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Trindade_e_Tobago",
                                    "http://en.wikipedia.org/wiki/Trinidad_and_Tobago"});
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Tunsia", "Tunisia"},
                             new String[] {"Tunis", "Tunis"},
                             +36, +48, +10, +11);
       loc[i].setFlag("tunisia.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Tun%C3%ADsia",
                                    "http://en.wikipedia.org/wiki/Tunisia"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Turquemenisto", "Turkmenistan"},
                             new String[] {"Ashgabat", "Ashgabat"},
                             +37, +58, +58, +24);
       loc[i].setFlag("turkmenistan.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Turquemenist%C3%A3o",
                                    "http://en.wikipedia.org/wiki/Turkmenistan"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Turquia", "Turkey"},
                             new String[] {"Ancara", "Ankara"},
                             +39, +55, +32, +55);
       loc[i].setFlag("turkey.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Turquia",
                                    "http://en.wikipedia.org/wiki/Turkey"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Ucrnia", "Ukraine"},
                             new String[] {"Kiev", "Kiev"},
                             +50, +27, +30, +30);
       loc[i].setFlag("ukraine.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Ucr%C3%A2nia",
                                    "http://en.wikipedia.org/wiki/Ukraine"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Uganda", "Uganda"},
                             new String[] {"Kampala", "Kampala"},
                             +0, +19, +32, +35);
       loc[i].setFlag("uganda.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Uganda",
                                    "http://en.wikipedia.org/wiki/Uganda"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Uruguai", "Uruguay"},
                             new String[] {"MonteVidu", "MonteVideo"},
                             -34, -53, -56, -10);
       loc[i].setFlag("uruguay.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Uruguai",
                                    "http://en.wikipedia.org/wiki/Uruguay"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Uzbequisto", "Uzbekistan"},
                             new String[] {"Tashkent", "Tashkent"},
                             +41, +16, +69, +13);
       loc[i].setFlag("uzbekistan.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Usbequist%C3%A3o",
                                    "http://en.wikipedia.org/wiki/Uzbekistan"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Vaticano", "Vatican City"},
                             new String[] {"Cidade do Vaticano", "Vatican City"},
                             +41, +54, +12, +27);
       loc[i].setFlag("vatican_city.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Cidade_do_Vaticano",
                                    "http://en.wikipedia.org/wiki/Vatican_City"});
       loc[i].setContinent("europe");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Venezuela", "Venezuela"},
                             new String[] {"Caracas", "Caracas"},
                             +9, +18, -66, -53);
       loc[i].setFlag("venezuela.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Venezuela",
                                    "http://en.wikipedia.org/wiki/Venezuela"});
       loc[i].setContinent("america");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Vietn", "Vietname"},
                             new String[] {"Hani", "Hanoi"},
                             +21, +1, +105, +52);
       loc[i].setFlag("vietname.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Vietname",
                                    "http://en.wikipedia.org/wiki/Vietnam"});
       loc[i].setContinent("asia");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Zmbia", "Zambia"},
                             new String[] {"Lusaka", "Lusaka"},
                             -15, -58, +25, +58);
       loc[i].setFlag("zambia.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Z%C3%A2mbia",
                                    "http://en.wikipedia.org/wiki/Zambia"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

       loc[i] = new Location(new String[] {"Zimbabwe", "Zimbabwe"},
                             new String[] {"Harare", "Harare"},
                             -17, -50, +31, +0);
       loc[i].setFlag("zimbabwe.gif");
       loc[i].setWiki(new String[] {"http://pt.wikipedia.org/wiki/Zimbabu%C3%A9",
                                    "http://en.wikipedia.org/wiki/Zimbabwe"});
       loc[i].setContinent("africa");
       locChoice.add(loc[i].getCountryName(language));
       i++;

/*
       // Sorting by capital names;
       boolean changed;
       do
       {
          changed = false;
          for (i = 0; i < number_of_cities - 1; i++)
          {
             if (loc[capital_sorted[i]].getCapital(language).compareToIgnoreCase(loc[capital_sorted[i + 1]].getCapital(language)) > 0)
             {
               int aux = capital_sorted[i];
               capital_sorted[i] = capital_sorted[i + 1];
               capital_sorted[i + 1] = aux;
               changed = true;
             };
          };
       } while (changed == true);

       for (i = 0; i < number_of_cities; i++)
       {
          System.out.println("capital_sorted["+ i + "] = " + capital_sorted[i] + ";");
       };
 */

       if (language == 0) // Portuguese
       {
           capital_sorted[0] = 54;
           capital_sorted[1] = 128;
           capital_sorted[2] = 70;
           capital_sorted[3] = 64;
           capital_sorted[4] = 96;
           capital_sorted[5] = 81;
           capital_sorted[6] = 186;
           capital_sorted[7] = 8;
           capital_sorted[8] = 109;
           capital_sorted[9] = 166;
           capital_sorted[10] = 11;
           capital_sorted[11] = 185;
           capital_sorted[12] = 56;
           capital_sorted[13] = 138;
           capital_sorted[14] = 37;
           capital_sorted[15] = 73;
           capital_sorted[16] = 88;
           capital_sorted[17] = 16;
           capital_sorted[18] = 112;
           capital_sorted[19] = 177;
           capital_sorted[20] = 26;
           capital_sorted[21] = 149;
           capital_sorted[22] = 69;
           capital_sorted[23] = 102;
           capital_sorted[24] = 168;
           capital_sorted[25] = 20;
           capital_sorted[26] = 119;
           capital_sorted[27] = 137;
           capital_sorted[28] = 5;
           capital_sorted[29] = 176;
           capital_sorted[30] = 148;
           capital_sorted[31] = 77;
           capital_sorted[32] = 159;
           capital_sorted[33] = 43;
           capital_sorted[34] = 25;
           capital_sorted[35] = 58;
           capital_sorted[36] = 151;
           capital_sorted[37] = 31;
           capital_sorted[38] = 157;
           capital_sorted[39] = 83;
           capital_sorted[40] = 10;
           capital_sorted[41] = 29;
           capital_sorted[42] = 52;
           capital_sorted[43] = 116;
           capital_sorted[44] = 15;
           capital_sorted[45] = 192;
           capital_sorted[46] = 121;
           capital_sorted[47] = 74;
           capital_sorted[48] = 169;
           capital_sorted[49] = 98;
           capital_sorted[50] = 118;
           capital_sorted[51] = 134;
           capital_sorted[52] = 191;
           capital_sorted[53] = 172;
           capital_sorted[54] = 76;
           capital_sorted[55] = 50;
           capital_sorted[56] = 115;
           capital_sorted[57] = 139;
           capital_sorted[58] = 19;
           capital_sorted[59] = 165;
           capital_sorted[60] = 170;
           capital_sorted[61] = 180;
           capital_sorted[62] = 51;
           capital_sorted[63] = 179;
           capital_sorted[64] = 147;
           capital_sorted[65] = 89;
           capital_sorted[66] = 178;
           capital_sorted[67] = 57;
           capital_sorted[68] = 162;
           capital_sorted[69] = 13;
           capital_sorted[70] = 175;
           capital_sorted[71] = 163;
           capital_sorted[72] = 38;
           capital_sorted[73] = 167;
           capital_sorted[74] = 24;
           capital_sorted[75] = 75;
           capital_sorted[76] = 72;
           capital_sorted[77] = 193;
           capital_sorted[78] = 195;
           capital_sorted[79] = 49;
           capital_sorted[80] = 66;
           capital_sorted[81] = 80;
           capital_sorted[82] = 136;
           capital_sorted[83] = 86;
           capital_sorted[84] = 92;
           capital_sorted[85] = 133;
           capital_sorted[86] = 140;
           capital_sorted[87] = 1;
           capital_sorted[88] = 188;
           capital_sorted[89] = 125;
           capital_sorted[90] = 174;
           capital_sorted[91] = 187;
           capital_sorted[92] = 160;
           capital_sorted[93] = 94;
           capital_sorted[94] = 150;
           capital_sorted[95] = 110;
           capital_sorted[96] = 23;
           capital_sorted[97] = 68;
           capital_sorted[98] = 111;
           capital_sorted[99] = 142;
           capital_sorted[100] = 146;
           capital_sorted[101] = 59;
           capital_sorted[102] = 182;
           capital_sorted[103] = 87;
           capital_sorted[104] = 9;
           capital_sorted[105] = 194;
           capital_sorted[106] = 107;
           capital_sorted[107] = 6;
           capital_sorted[108] = 3;
           capital_sorted[109] = 60;
           capital_sorted[110] = 78;
           capital_sorted[111] = 18;
           capital_sorted[112] = 7;
           capital_sorted[113] = 65;
           capital_sorted[114] = 126;
           capital_sorted[115] = 120;
           capital_sorted[116] = 100;
           capital_sorted[117] = 173;
           capital_sorted[118] = 22;
           capital_sorted[119] = 171;
           capital_sorted[120] = 103;
           capital_sorted[121] = 189;
           capital_sorted[122] = 161;
           capital_sorted[123] = 132;
           capital_sorted[124] = 97;
           capital_sorted[125] = 154;
           capital_sorted[126] = 127;
           capital_sorted[127] = 42;
           capital_sorted[128] = 117;
           capital_sorted[129] = 130;
           capital_sorted[130] = 85;
           capital_sorted[131] = 39;
           capital_sorted[132] = 129;
           capital_sorted[133] = 36;
           capital_sorted[134] = 28;
           capital_sorted[135] = 181;
           capital_sorted[136] = 67;
           capital_sorted[137] = 41;
           capital_sorted[138] = 35;
           capital_sorted[139] = 135;
           capital_sorted[140] = 183;
           capital_sorted[141] = 155;
           capital_sorted[142] = 21;
           capital_sorted[143] = 79;
           capital_sorted[144] = 158;
           capital_sorted[145] = 153;
           capital_sorted[146] = 33;
           capital_sorted[147] = 2;
           capital_sorted[148] = 123;
           capital_sorted[149] = 44;
           capital_sorted[150] = 55;
           capital_sorted[151] = 114;
           capital_sorted[152] = 141;
           capital_sorted[153] = 91;
           capital_sorted[154] = 12;
           capital_sorted[155] = 101;
           capital_sorted[156] = 0;
           capital_sorted[157] = 156;
           capital_sorted[158] = 93;
           capital_sorted[159] = 17;
           capital_sorted[160] = 46;
           capital_sorted[161] = 145;
           capital_sorted[162] = 53;
           capital_sorted[163] = 84;
           capital_sorted[164] = 40;
           capital_sorted[165] = 152;
           capital_sorted[166] = 32;
           capital_sorted[167] = 45;
           capital_sorted[168] = 108;
           capital_sorted[169] = 27;
           capital_sorted[170] = 113;
           capital_sorted[171] = 164;
           capital_sorted[172] = 63;
           capital_sorted[173] = 190;
           capital_sorted[174] = 90;
           capital_sorted[175] = 82;
           capital_sorted[176] = 143;
           capital_sorted[177] = 30;
           capital_sorted[178] = 71;
           capital_sorted[179] = 4;
           capital_sorted[180] = 104;
           capital_sorted[181] = 184;
           capital_sorted[182] = 95;
           capital_sorted[183] = 122;
           capital_sorted[184] = 105;
           capital_sorted[185] = 144;
           capital_sorted[186] = 14;
           capital_sorted[187] = 99;
           capital_sorted[188] = 106;
           capital_sorted[189] = 61;
           capital_sorted[190] = 62;
           capital_sorted[191] = 131;
           capital_sorted[192] = 124;
           capital_sorted[193] = 47;
           capital_sorted[194] = 34;
           capital_sorted[195] = 48;
       };

       if (language == 1) // English
       {
           capital_sorted[0] = 54;
           capital_sorted[1] = 128;
           capital_sorted[2] = 70;
           capital_sorted[3] = 64;
           capital_sorted[4] = 11;
           capital_sorted[5] = 96;
           capital_sorted[6] = 81;
           capital_sorted[7] = 8;
           capital_sorted[8] = 186;
           capital_sorted[9] = 109;
           capital_sorted[10] = 166;
           capital_sorted[11] = 185;
           capital_sorted[12] = 56;
           capital_sorted[13] = 37;
           capital_sorted[14] = 138;
           capital_sorted[15] = 73;
           capital_sorted[16] = 88;
           capital_sorted[17] = 16;
           capital_sorted[18] = 112;
           capital_sorted[19] = 26;
           capital_sorted[20] = 177;
           capital_sorted[21] = 149;
           capital_sorted[22] = 69;
           capital_sorted[23] = 41;
           capital_sorted[24] = 102;
           capital_sorted[25] = 137;
           capital_sorted[26] = 168;
           capital_sorted[27] = 20;
           capital_sorted[28] = 119;
           capital_sorted[29] = 5;
           capital_sorted[30] = 176;
           capital_sorted[31] = 148;
           capital_sorted[32] = 77;
           capital_sorted[33] = 159;
           capital_sorted[34] = 43;
           capital_sorted[35] = 25;
           capital_sorted[36] = 58;
           capital_sorted[37] = 151;
           capital_sorted[38] = 31;
           capital_sorted[39] = 157;
           capital_sorted[40] = 83;
           capital_sorted[41] = 10;
           capital_sorted[42] = 29;
           capital_sorted[43] = 52;
           capital_sorted[44] = 15;
           capital_sorted[45] = 116;
           capital_sorted[46] = 192;
           capital_sorted[47] = 121;
           capital_sorted[48] = 172;
           capital_sorted[49] = 76;
           capital_sorted[50] = 50;
           capital_sorted[51] = 115;
           capital_sorted[52] = 139;
           capital_sorted[53] = 165;
           capital_sorted[54] = 170;
           capital_sorted[55] = 19;
           capital_sorted[56] = 180;
           capital_sorted[57] = 51;
           capital_sorted[58] = 179;
           capital_sorted[59] = 147;
           capital_sorted[60] = 89;
           capital_sorted[61] = 178;
           capital_sorted[62] = 57;
           capital_sorted[63] = 162;
           capital_sorted[64] = 163;
           capital_sorted[65] = 38;
           capital_sorted[66] = 167;
           capital_sorted[67] = 24;
           capital_sorted[68] = 75;
           capital_sorted[69] = 72;
           capital_sorted[70] = 74;
           capital_sorted[71] = 193;
           capital_sorted[72] = 195;
           capital_sorted[73] = 49;
           capital_sorted[74] = 66;
           capital_sorted[75] = 80;
           capital_sorted[76] = 136;
           capital_sorted[77] = 86;
           capital_sorted[78] = 92;
           capital_sorted[79] = 133;
           capital_sorted[80] = 140;
           capital_sorted[81] = 1;
           capital_sorted[82] = 188;
           capital_sorted[83] = 125;
           capital_sorted[84] = 174;
           capital_sorted[85] = 187;
           capital_sorted[86] = 160;
           capital_sorted[87] = 94;
           capital_sorted[88] = 150;
           capital_sorted[89] = 110;
           capital_sorted[90] = 98;
           capital_sorted[91] = 23;
           capital_sorted[92] = 68;
           capital_sorted[93] = 111;
           capital_sorted[94] = 142;
           capital_sorted[95] = 146;
           capital_sorted[96] = 59;
           capital_sorted[97] = 182;
           capital_sorted[98] = 87;
           capital_sorted[99] = 9;
           capital_sorted[100] = 194;
           capital_sorted[101] = 107;
           capital_sorted[102] = 6;
           capital_sorted[103] = 3;
           capital_sorted[104] = 60;
           capital_sorted[105] = 78;
           capital_sorted[106] = 126;
           capital_sorted[107] = 18;
           capital_sorted[108] = 7;
           capital_sorted[109] = 65;
           capital_sorted[110] = 120;
           capital_sorted[111] = 100;
           capital_sorted[112] = 173;
           capital_sorted[113] = 118;
           capital_sorted[114] = 22;
           capital_sorted[115] = 171;
           capital_sorted[116] = 103;
           capital_sorted[117] = 189;
           capital_sorted[118] = 161;
           capital_sorted[119] = 132;
           capital_sorted[120] = 97;
           capital_sorted[121] = 154;
           capital_sorted[122] = 85;
           capital_sorted[123] = 127;
           capital_sorted[124] = 42;
           capital_sorted[125] = 117;
           capital_sorted[126] = 130;
           capital_sorted[127] = 39;
           capital_sorted[128] = 129;
           capital_sorted[129] = 36;
           capital_sorted[130] = 28;
           capital_sorted[131] = 181;
           capital_sorted[132] = 134;
           capital_sorted[133] = 67;
           capital_sorted[134] = 35;
           capital_sorted[135] = 135;
           capital_sorted[136] = 183;
           capital_sorted[137] = 79;
           capital_sorted[138] = 155;
           capital_sorted[139] = 158;
           capital_sorted[140] = 21;
           capital_sorted[141] = 153;
           capital_sorted[142] = 33;
           capital_sorted[143] = 2;
           capital_sorted[144] = 123;
           capital_sorted[145] = 44;
           capital_sorted[146] = 55;
           capital_sorted[147] = 114;
           capital_sorted[148] = 141;
           capital_sorted[149] = 91;
           capital_sorted[150] = 101;
           capital_sorted[151] = 0;
           capital_sorted[152] = 156;
           capital_sorted[153] = 12;
           capital_sorted[154] = 93;
           capital_sorted[155] = 17;
           capital_sorted[156] = 46;
           capital_sorted[157] = 145;
           capital_sorted[158] = 53;
           capital_sorted[159] = 84;
           capital_sorted[160] = 40;
           capital_sorted[161] = 152;
           capital_sorted[162] = 164;
           capital_sorted[163] = 32;
           capital_sorted[164] = 45;
           capital_sorted[165] = 169;
           capital_sorted[166] = 108;
           capital_sorted[167] = 27;
           capital_sorted[168] = 175;
           capital_sorted[169] = 113;
           capital_sorted[170] = 71;
           capital_sorted[171] = 63;
           capital_sorted[172] = 190;
           capital_sorted[173] = 82;
           capital_sorted[174] = 90;
           capital_sorted[175] = 143;
           capital_sorted[176] = 30;
           capital_sorted[177] = 4;
           capital_sorted[178] = 95;
           capital_sorted[179] = 104;
           capital_sorted[180] = 184;
           capital_sorted[181] = 122;
           capital_sorted[182] = 105;
           capital_sorted[183] = 191;
           capital_sorted[184] = 14;
           capital_sorted[185] = 99;
           capital_sorted[186] = 106;
           capital_sorted[187] = 61;
           capital_sorted[188] = 144;
           capital_sorted[189] = 62;
           capital_sorted[190] = 131;
           capital_sorted[191] = 124;
           capital_sorted[192] = 47;
           capital_sorted[193] = 34;
           capital_sorted[194] = 13;
           capital_sorted[195] = 48;
       };
   }
}; // End of PjEarth_IsP class


///////////////////////////////////////////////////////////////////////////////
// Drawing panel class for holding images.
///////////////////////////////////////////////////////////////////////////////
class DrawingPanel extends Panel
{
    protected Image img = null;
    protected int pnl_w;
    int img_w;
    int img_h;

    void DrawingPanel()
    {
       img = null;
       pnl_w = img_w = img_h = -1;
    };

    void setImage(Image image, int input_pnl_w)
    {
       img = image;
       pnl_w = input_pnl_w;

       img_w = img.getWidth(this);
       img_h = img.getHeight(this);
   };

    public void paint(Graphics g)
    {
       super.paint(g);

       if (img != null)
       {
           int imgX = (pnl_w - img_w)/2;
           int imgY = 0;

           this.setSize(pnl_w, img_h);

           g.drawImage(img, imgX, imgY, this);
       };
    };
}; // End of DrawingPanel class


